@@ -7,9 +7,10 @@ from pprint import pprint
 import sys
 import warnings
 import traceback
+from .tracing import trace, event
 
 from numba import (bytecode, interpreter, funcdesc, typing, typeinfer,
-                   lowering, objmode, irpasses, utils, config,
+                   lowering, objmode, irpasses, utils, config, errors,
                    types, ir, looplifting, macro, types, rewrites)
 from numba.targets import cpu, callconv
 from numba.annotations import type_annotations
@@ -166,7 +167,7 @@ def compile_result(**kws):
 def compile_isolated(func, args, return_type=None, flags=DEFAULT_FLAGS,
                      locals={}):
     """
-    Compile the function is an isolated environment.
+    Compile the function in an isolated environment (typing and target context).
     Good for testing.
     """
     typingctx = typing.Context()
@@ -237,10 +238,12 @@ class _PipelineManager(object):
         assert self._finalized, "PM must be finalized before run()"
         res = None
         for pipeline_name in self.pipeline_order:
+            event(pipeline_name)
             is_final_pipeline = pipeline_name == self.pipeline_order[-1]
             for stage, stage_name in self.pipeline_stages[pipeline_name]:
                 try:
-                    res = stage()
+                    event(stage_name)
+                    stage()
                 except _EarlyPipelineCompletion as e:
                     return e.result
                 except BaseException as e:
@@ -254,7 +257,7 @@ class _PipelineManager(object):
                         status.fail_reason = patched_exception
                         break
             else:
-                return res
+                return None
 
         # TODO save all error information
         raise CompilerError("All pipelines have failed")
@@ -287,12 +290,17 @@ class Pipeline(object):
         self.return_type = return_type
         self.flags = flags
         self.locals = locals
+
         # Results of various steps of the compilation pipeline
+        self.interp = None
         self.bc = None
+        self.func = None
         self.func_attr = None
         self.lifted = None
+        self.lifted_from = None
         self.typemap = None
         self.calltypes = None
+        self.type_annotation = None
 
         self.status = _CompileStatus(
             can_fallback=self.flags.enable_pyobject,
@@ -314,7 +322,7 @@ class Pipeline(object):
                     # Clear all references attached to the traceback
                     e = e.with_traceback(None)
                 warnings.warn_explicit('%s: %s' % (msg, e),
-                                       config.NumbaWarning,
+                                       errors.NumbaWarning,
                                        self.func_attr.filename,
                                        self.func_attr.lineno)
 
@@ -335,7 +343,7 @@ class Pipeline(object):
                     # Clear all references attached to the traceback
                     e = e.with_traceback(None)
                 warnings.warn_explicit('%s: %s' % (msg, e),
-                                       config.NumbaWarning,
+                                       errors.NumbaWarning,
                                        self.func_attr.filename,
                                        self.func_attr.lineno)
 
@@ -359,7 +367,8 @@ class Pipeline(object):
             bc = self.extract_bytecode(func)
         except BaseException as e:
             if self.status.can_giveup:
-                return self.stage_compile_interp_mode()
+                self.stage_compile_interp_mode()
+                return self.cr
             else:
                 raise e
 
@@ -558,29 +567,28 @@ class Pipeline(object):
 
         lowered = lowerfn()
         signature = typing.signature(self.return_type, *self.args)
-        cr = compile_result(typing_context=self.typingctx,
-                            target_context=self.targetctx,
-                            entry_point=lowered.cfunc,
-                            typing_error=self.status.fail_reason,
-                            type_annotation=self.type_annotation,
-                            library=self.library,
-                            call_helper=lowered.call_helper,
-                            signature=signature,
-                            objectmode=objectmode,
-                            interpmode=False,
-                            lifted=self.lifted,
-                            fndesc=lowered.fndesc,
-                            environment=lowered.env,
-                            has_dynamic_globals=lowered.has_dynamic_globals,
-                            )
-        return cr
+        self.cr = compile_result(typing_context=self.typingctx,
+                                 target_context=self.targetctx,
+                                 entry_point=lowered.cfunc,
+                                 typing_error=self.status.fail_reason,
+                                 type_annotation=self.type_annotation,
+                                 library=self.library,
+                                 call_helper=lowered.call_helper,
+                                 signature=signature,
+                                 objectmode=objectmode,
+                                 interpmode=False,
+                                 lifted=self.lifted,
+                                 fndesc=lowered.fndesc,
+                                 environment=lowered.env,
+                                 has_dynamic_globals=lowered.has_dynamic_globals,
+                                 )
 
     def stage_objectmode_backend(self):
         """
         Lowering for object mode
         """
         lowerfn = self.backend_object_mode
-        res = self._backend(lowerfn, objectmode=True)
+        self._backend(lowerfn, objectmode=True)
 
         # Warn if compiled function in object mode and force_pyobject not set
         if not self.flags.force_pyobject:
@@ -588,22 +596,21 @@ class Pipeline(object):
                 warn_msg = 'Function "%s" was compiled in object mode without forceobj=True, but has lifted loops.' % (self.func_attr.name,)
             else:
                 warn_msg = 'Function "%s" was compiled in object mode without forceobj=True.' % (self.func_attr.name,)
-            warnings.warn_explicit(warn_msg, config.NumbaWarning,
+            warnings.warn_explicit(warn_msg, errors.NumbaWarning,
                                    self.func_attr.filename,
                                    self.func_attr.lineno)
             if self.flags.release_gil:
                 warn_msg = "Code running in object mode won't allow parallel execution despite nogil=True."
-                warnings.warn_explicit(warn_msg, config.NumbaWarning,
+                warnings.warn_explicit(warn_msg, errors.NumbaWarning,
                                        self.func_attr.filename,
                                        self.func_attr.lineno)
-        return res
 
     def stage_nopython_backend(self):
         """
         Do lowering for nopython
         """
         lowerfn = self.backend_nopython_mode
-        return self._backend(lowerfn, objectmode=False)
+        self._backend(lowerfn, objectmode=False)
 
     def stage_compile_interp_mode(self):
         """
@@ -611,17 +618,23 @@ class Pipeline(object):
         """
         args = [types.pyobject] * len(self.args)
         signature = typing.signature(types.pyobject, *args)
-        cr = compile_result(typing_context=self.typingctx,
-                            target_context=self.targetctx,
-                            entry_point=self.func,
-                            typing_error=self.status.fail_reason,
-                            type_annotation="<Interpreter mode function>",
-                            signature=signature,
-                            objectmode=False,
-                            interpmode=True,
-                            lifted=(),
-                            fndesc=None,)
-        return cr
+        self.cr = compile_result(typing_context=self.typingctx,
+                                 target_context=self.targetctx,
+                                 entry_point=self.func,
+                                 typing_error=self.status.fail_reason,
+                                 type_annotation="<Interpreter mode function>",
+                                 signature=signature,
+                                 objectmode=False,
+                                 interpmode=True,
+                                 lifted=(),
+                                 fndesc=None,)
+
+    def stage_cleanup(self):
+        """
+        Cleanup intermediate results to release resources.
+        """
+        if self.interp is not None:
+            self.interp.reset()
 
     def _compile_bytecode(self):
         pm = _PipelineManager()
@@ -636,6 +649,7 @@ class Pipeline(object):
             if not self.flags.no_rewrites:
                 pm.add_stage(self.stage_nopython_rewrites, "nopython rewrites")
             pm.add_stage(self.stage_nopython_backend, "nopython mode backend")
+            pm.add_stage(self.stage_cleanup, "cleanup intermediate results")
 
         if self.status.can_fallback or self.flags.force_pyobject:
             pm.create_pipeline("object")
@@ -643,13 +657,21 @@ class Pipeline(object):
             pm.add_stage(self.stage_objectmode_frontend, "object mode frontend")
             pm.add_stage(self.stage_annotate_type, "annotate type")
             pm.add_stage(self.stage_objectmode_backend, "object mode backend")
+            pm.add_stage(self.stage_cleanup, "cleanup intermediate results")
 
         if self.status.can_giveup:
             pm.create_pipeline("interp")
             pm.add_stage(self.stage_compile_interp_mode, "compiling with interpreter mode")
+            pm.add_stage(self.stage_cleanup, "cleanup intermediate results")
 
         pm.finalize()
-        return pm.run(self.status)
+        res = pm.run(self.status)
+        if res is not None:
+            # Early pipeline completion
+            return res
+        else:
+            assert self.cr is not None
+            return self.cr
 
 
 def compile_extra(typingctx, targetctx, func, args, return_type, flags,
@@ -762,7 +784,8 @@ def type_inference_stage(typingctx, interp, args, return_type, locals={}):
     if len(args) != interp.arg_count:
         raise TypeError("Mismatch number of argument types")
 
-    infer = typeinfer.TypeInferer(typingctx, interp)
+    warnings = errors.WarningsFixer(errors.NumbaWarning)
+    infer = typeinfer.TypeInferer(typingctx, interp, warnings)
 
     # Seed argument types
     for index, (name, ty) in enumerate(zip(interp.bytecode.arg_names, args)):
@@ -780,6 +803,9 @@ def type_inference_stage(typingctx, interp, args, return_type, locals={}):
     infer.propagate()
     typemap, restype, calltypes = infer.unify()
 
+    # Output all Numba warnings
+    warnings.flush()
+
     return typemap, restype, calltypes
 
 
