@@ -8,7 +8,6 @@ import warnings
 from . import config, sigutils
 from .errors import DeprecationError
 from .targets import registry
-from . import cuda
 
 # -----------------------------------------------------------------------------
 # Decorators
@@ -52,11 +51,11 @@ def jit(signature_or_function=None, locals={}, target='cpu', cache=False, **opti
         Mapping of local variable names to Numba types. Used to override the
         types deduced by Numba's type inference engine.
 
-    targets: str
+    target: str
         Specifies the target platform to compile for. Valid targets are cpu,
         gpu, npyufunc, and cuda. Defaults to cpu.
 
-    targetoptions: 
+    targetoptions:
         For a cpu target, valid options are:
             nopython: bool
                 Set to True to disable the use of PyObjects and Python API
@@ -151,16 +150,18 @@ def jit(signature_or_function=None, locals={}, target='cpu', cache=False, **opti
         return wrapper
 
 
-def _jit(sigs, locals, target, cache, targetoptions):
+def _jit(sigs, locals, target, cache, targetoptions, **dispatcher_args):
     dispatcher = registry.dispatcher_registry[target]
 
     def wrapper(func):
         if config.ENABLE_CUDASIM and target == 'cuda':
+            from . import cuda
             return cuda.jit(func)
         if config.DISABLE_JIT and not target == 'npyufunc':
             return _DisableJitWrapper(func)
         disp = dispatcher(py_func=func, locals=locals,
-                          targetoptions=targetoptions)
+                          targetoptions=targetoptions,
+                          **dispatcher_args)
         if cache:
             disp.enable_caching()
         if sigs is not None:
@@ -172,6 +173,21 @@ def _jit(sigs, locals, target, cache, targetoptions):
     return wrapper
 
 
+def generated_jit(function=None, target='cpu', cache=False, **options):
+    """
+    This decorator allows flexible type-based compilation
+    of a jitted function.  It works as `@jit`, except that the decorated
+    function is called at compile-time with the *types* of the arguments
+    and should return an implementation function for those types.
+    """
+    wrapper = _jit(sigs=None, locals={}, target=target, cache=cache,
+                   targetoptions=options, impl_kind='generated')
+    if function is not None:
+        return wrapper(function)
+    else:
+        return wrapper
+
+
 def njit(*args, **kws):
     """
     Equivalent to jit(nopython=True)
