@@ -142,9 +142,9 @@ class BaseLower(object):
         self.library.add_ir_module(self.module)
 
     def extract_function_arguments(self):
-        rawfnargs = self.call_conv.get_arguments(self.function)
-        arginfo = self.context.get_arg_packer(self.fndesc.argtypes)
-        self.fnargs = arginfo.from_arguments(self.builder, rawfnargs)
+        self.fnargs = self.call_conv.decode_arguments(self.builder,
+                                                      self.fndesc.argtypes,
+                                                      self.function)
         return self.fnargs
 
     def lower_normal_function(self, fndesc):
@@ -307,14 +307,7 @@ class Lower(BaseLower):
             return impl(self.builder, (target, index))
 
         elif isinstance(inst, ir.Del):
-            try:
-                # XXX: incorrect Del injection?
-                val = self.loadvar(inst.value)
-            except KeyError:
-                pass
-            else:
-                self.decref(self.typeof(inst.value), val)
-                self._delete_variable(inst.value)
+            self.delvar(inst.value)
 
         elif isinstance(inst, ir.SetAttr):
             target = self.loadvar(inst.target.name)
@@ -394,11 +387,16 @@ class Lower(BaseLower):
             return res
 
         elif isinstance(value, ir.Arg):
-            val = self.fnargs[value.index]
             # Cast from the argument type to the local variable type
             # (note the "arg.FOO" convention as used in typeinfer)
-            oty = self.typeof("arg." + value.name)
-            res = self.context.cast(self.builder, val, oty, ty)
+            argty = self.typeof("arg." + value.name)
+            if isinstance(argty, types.Omitted):
+                pyval = argty.value
+                res = self.context.get_constant_generic(self.builder, ty,
+                                                        pyval)
+            else:
+                val = self.fnargs[value.index]
+                res = self.context.cast(self.builder, val, argty, ty)
             self.incref(ty, res)
             return res
 
@@ -767,17 +765,10 @@ class Lower(BaseLower):
 
         raise NotImplementedError(expr)
 
-    def getvar(self, name):
-        return self.varmap[name]
-
-    def loadvar(self, name):
-        ptr = self.getvar(name)
-        return self.builder.load(ptr)
-
-    def storevar(self, value, name):
-        fetype = self.typeof(name)
-
-        # Define if not already
+    def _alloca_var(self, name, fetype):
+        """
+        Ensure the given variable has an allocated stack slot.
+        """
         if name not in self.varmap:
             # If not already defined, allocate it
             llty = self.context.get_value_type(fetype)
@@ -785,6 +776,28 @@ class Lower(BaseLower):
             # Remember the pointer
             self.varmap[name] = ptr
 
+    def getvar(self, name):
+        """
+        Get a pointer to the given variable's slot.
+        """
+        return self.varmap[name]
+
+    def loadvar(self, name):
+        """
+        Load the given variable's value.
+        """
+        ptr = self.getvar(name)
+        return self.builder.load(ptr)
+
+    def storevar(self, value, name):
+        """
+        Store the value into the given variable.
+        """
+        fetype = self.typeof(name)
+
+        # Define if not already
+        self._alloca_var(name, fetype)
+
         # Clean up existing value stored in the variable
         old = self.loadvar(name)
         self.decref(fetype, old)
@@ -799,6 +812,21 @@ class Lower(BaseLower):
 
         self.builder.store(value, ptr)
 
+    def delvar(self, name):
+        """
+        Delete the given variable.
+        """
+        fetype = self.typeof(name)
+
+        # Define if not already (may happen if the variable is deleted
+        # at the beginning of a loop, but only set later in the loop)
+        self._alloca_var(name, fetype)
+
+        ptr = self.getvar(name)
+        self.decref(fetype, self.builder.load(ptr))
+        # Zero-fill variable to avoid double frees on subsequent dels
+        self.builder.store(Constant.null(ptr.type.pointee), ptr)
+
     def alloca(self, name, type):
         lltype = self.context.get_value_type(type)
         return self.alloca_lltype(name, lltype)
@@ -817,10 +845,3 @@ class Lower(BaseLower):
             return
 
         self.context.nrt_decref(self.builder, typ, val)
-
-    def _delete_variable(self, varname):
-        """
-        Zero-fill variable to avoid crashing due to extra ir.Del
-        """
-        storage = self.getvar(varname)
-        self.builder.store(Constant.null(storage.type.pointee), storage)
