@@ -663,16 +663,16 @@ class PythonAPI(object):
     # Concrete slice API
     #
 
-    def slice_as_ints(self, obj, defaults):
+    def slice_as_ints(self, obj):
         """
         Read the members of a slice of integers.
+
         Returns a (ok, start, stop, step) tuple where ok is a boolean and
         the following members are pointer-sized ints.
         """
-        defaults = [ir.Constant(self.py_ssize_t, v) for v in defaults]
-        pstart = cgutils.alloca_once_value(self.builder, defaults[0])
-        pstop = cgutils.alloca_once_value(self.builder, defaults[1])
-        pstep = cgutils.alloca_once_value(self.builder, defaults[2])
+        pstart = cgutils.alloca_once(self.builder, self.py_ssize_t)
+        pstop = cgutils.alloca_once(self.builder, self.py_ssize_t)
+        pstep = cgutils.alloca_once(self.builder, self.py_ssize_t)
         fnty = Type.function(Type.int(),
                              [self.pyobj] + [self.py_ssize_t.as_pointer()] * 3)
         fn = self._get_function(fnty, name="numba_unpack_slice")
@@ -872,6 +872,16 @@ class PythonAPI(object):
         args.append(self.context.get_constant_null(types.pyobject))
         return self.builder.call(fn, args)
 
+    def call_method(self, callee, method, objargs = None):
+        cstr = self.context.insert_const_string(self.module, method)
+        fnty = Type.function(self.pyobj, [self.pyobj, self.cstring], var_arg=True)
+        fn = self._get_function(fnty, name="PyObject_CallMethod")
+        args = [callee, cstr]
+        if objargs:
+            args.extend(objargs)
+        args.append(self.context.get_constant_null(types.pyobject))
+        return self.builder.call(fn, args)
+
     def call(self, callee, args=None, kws=None):
         if args is None:
             args = self.get_null_object()
