@@ -3,6 +3,8 @@ from __future__ import print_function, absolute_import, division
 import math
 from functools import reduce
 
+import numpy
+
 from llvmlite import ir
 from llvmlite.llvmpy.core import Type, Constant
 import llvmlite.llvmpy.core as lc
@@ -11,7 +13,7 @@ from .imputils import (lower_builtin, lower_getattr, lower_getattr_generic,
                        lower_cast, iternext_impl,
                        impl_ret_borrowed, impl_ret_untracked)
 from . import optional
-from .. import typing, types, cgutils, utils, intrinsics
+from .. import typing, types, cgutils, utils
 
 
 @lower_builtin('is not', types.Any, types.Any)
@@ -1327,11 +1329,17 @@ def complex_impl(context, builder, sig, args):
 @lower_builtin(types.NumberClass, types.Any)
 def number_constructor(context, builder, sig, args):
     """
-    Convert any number to any other.
+    Call a number class, e.g. np.int32(...)
     """
-    [val] = args
-    [valty] = sig.args
-    return context.cast(builder, val, valty, sig.return_type)
+    if isinstance(sig.return_type, types.Array):
+        # Array constructor
+        impl = context.get_function(numpy.array, sig)
+        return impl(builder, args)
+    else:
+        # Scalar constructor
+        [val] = args
+        [valty] = sig.args
+        return context.cast(builder, val, valty, sig.return_type)
 
 
 #-------------------------------------------------------------------------------
@@ -1412,37 +1420,6 @@ def boolean_to_any(context, builder, fromty, toty, val):
     asint = builder.zext(val, Type.int())
     return context.cast(builder, asint, types.int32, toty)
 
-
-# -----------------------------------------------------------------------------
-
-@lower_builtin(intrinsics.array_ravel, types.Array)
-def array_ravel_impl(context, builder, sig, args):
-    [arrty] = sig.args
-    [arr] = args
-    flatarrty = sig.return_type
-
-    flatarrcls = context.make_array(flatarrty)
-    arrcls = context.make_array(arrty)
-
-    flatarr = flatarrcls(context, builder)
-    arr = arrcls(context, builder, value=arr)
-
-    shapes = cgutils.unpack_tuple(builder, arr.shape, arrty.ndim)
-    size = reduce(builder.mul, shapes)
-    strides = cgutils.unpack_tuple(builder, arr.strides, arrty.ndim)
-    unit_stride = strides[0] if arrty.layout == 'F' else strides[-1]
-
-    context.populate_array(flatarr,
-                           data=arr.data,
-                           shape=cgutils.pack_array(builder, [size]),
-                           strides=cgutils.pack_array(builder, [unit_stride]),
-                           itemsize=arr.itemsize,
-                           parent=arr.parent)
-
-    res = flatarr._getvalue()
-    return impl_ret_borrowed(context, builder, sig.return_type, res)
-
-
 # -----------------------------------------------------------------------------
 
 @lower_builtin(type, types.Any)
