@@ -1,6 +1,7 @@
 from __future__ import print_function, division, absolute_import
 
 import functools
+import locale
 import sys
 import weakref
 
@@ -160,6 +161,9 @@ class CodeLibrary(object):
         Finalization involves various stages of code optimization and
         linking.
         """
+        # Report any LLVM-related problems to the user
+        self._codegen._check_llvm_bugs()
+
         self._raise_if_finalized()
 
         if config.DUMP_FUNC_OPT:
@@ -399,14 +403,13 @@ class BaseCPUCodegen(object):
     def _init(self, llvm_module):
         assert list(llvm_module.global_variables) == [], "Module isn't empty"
 
-        target = ll.Target.from_default_triple()
-        tm_options = dict(cpu='', features='', opt=config.OPT)
+        target = ll.Target.from_triple(ll.get_process_triple())
+        tm_options = dict(opt=config.OPT)
+        self._tm_features = self._customize_tm_features()
         self._customize_tm_options(tm_options)
         tm = target.create_target_machine(**tm_options)
         engine = ll.create_mcjit_compiler(llvm_module, tm)
-        tli = ll.create_target_library_info(llvm_module.triple)
 
-        self._tli = tli
         self._tm = tm
         self._engine = engine
         self._target_data = engine.target_data
@@ -418,7 +421,7 @@ class BaseCPUCodegen(object):
 
     def _create_empty_module(self, name):
         ir_module = lc.Module.new(name)
-        ir_module.triple = ll.get_default_triple()
+        ir_module.triple = ll.get_process_triple()
         if self._data_layout:
             ir_module.data_layout = self._data_layout
         return ir_module
@@ -452,7 +455,6 @@ class BaseCPUCodegen(object):
         pm = ll.create_module_pass_manager()
         dl = ll.create_target_data(self._data_layout)
         dl.add_pass(pm)
-        self._tli.add_pass(pm)
         self._tm.add_analysis_passes(pm)
         with self._pass_manager_builder() as pmb:
             pmb.populate(pm)
@@ -461,7 +463,6 @@ class BaseCPUCodegen(object):
     def _function_pass_manager(self, llvm_module):
         pm = ll.create_function_pass_manager(llvm_module)
         self._target_data.add_pass(pm)
-        self._tli.add_pass(pm)
         self._tm.add_analysis_passes(pm)
         with self._pass_manager_builder() as pmb:
             pmb.populate(pm)
@@ -480,12 +481,42 @@ class BaseCPUCodegen(object):
             opt=config.OPT, loop_vectorize=config.LOOP_VECTORIZE)
         return pmb
 
+    def _check_llvm_bugs(self):
+        """
+        Guard against some well-known LLVM bug(s).
+        """
+        # Check the locale bug at https://github.com/numba/numba/issues/1569
+        # Note we can't cache the result as locale settings can change
+        # accross a process's lifetime.  Also, for this same reason,
+        # the check here is a mere heuristic (there may be a race condition
+        # between now and actually compiling IR).
+        ir = """
+            define double @func()
+            {
+                ret double 1.23e+01
+            }
+            """
+        mod = ll.parse_assembly(ir)
+        ir_out = str(mod)
+        if "12.3" in ir_out or "1.23" in ir_out:
+            # Everything ok
+            return
+        if "1.0" in ir_out:
+            loc = locale.getlocale()
+            raise RuntimeError(
+                "LLVM will produce incorrect floating-point code "
+                "in the current locale %s.\nPlease read "
+                "http://numba.pydata.org/numba-doc/dev/user/faq.html#llvm-locale-bug "
+                "for more information."
+                % (loc,))
+        raise AssertionError("Unexpected IR:\n%s\n" % (ir_out,))
+
     def magic_tuple(self):
         """
         Return a tuple unambiguously describing the codegen behaviour.
         """
         return (self._llvm_module.triple, ll.get_host_cpu_name(),
-                config.ENABLE_AVX)
+                self._tm_features)
 
 
 class AOTCPUCodegen(BaseCPUCodegen):
@@ -497,8 +528,13 @@ class AOTCPUCodegen(BaseCPUCodegen):
     _library_class = AOTCodeLibrary
 
     def _customize_tm_options(self, options):
+        options['cpu'] = ''  # use generic cpu model for the arch
         options['reloc'] = 'pic'
         options['codemodel'] = 'default'
+        options['features'] = self._tm_features
+
+    def _customize_tm_features(self):
+        return ''
 
     def _add_module(self, module):
         pass
@@ -512,8 +548,6 @@ class JITCPUCodegen(BaseCPUCodegen):
     _library_class = JITCodeLibrary
 
     def _customize_tm_options(self, options):
-        features = []
-
         # As long as we don't want to ship the code to another machine,
         # we can specialize for this CPU.
         options['cpu'] = ll.get_host_cpu_name()
@@ -521,17 +555,27 @@ class JITCPUCodegen(BaseCPUCodegen):
         options['reloc'] = 'default'
         options['codemodel'] = 'jitdefault'
 
+        # Set feature attributes
+        options['features'] = self._tm_features
+
+        # Enable JIT debug
+        options['jitdebug'] = True
+
+    def _customize_tm_features(self):
+        # For JIT target, we will use LLVM to get the feature map
+        features = ll.get_host_cpu_features()
+
         # There are various performance issues with AVX and LLVM 3.5
         # (list at http://llvm.org/bugs/buglist.cgi?quicksearch=avx).
         # For now we'd rather disable it, since it can pessimize the code.
         if not config.ENABLE_AVX:
-            features.append('-avx')
+            # Disable all feature with name starting with 'avx'
+            for k in features:
+                if k.startswith('avx'):
+                    features[k] = False
 
         # Set feature attributes
-        options['features'] = ','.join(features)
-
-        # Enable JIT debug
-        options['jitdebug'] = True
+        return features.flatten()
 
     def _add_module(self, module):
         self._engine.add_module(module)
