@@ -8,8 +8,7 @@ import math
 
 from llvmlite import ir
 from numba import types, cgutils, typing
-from numba.targets.imputils import (builtin, builtin_attr, implement,
-                                    impl_attribute, impl_attribute_generic,
+from numba.targets.imputils import (lower_builtin, lower_cast,
                                     iternext_impl, impl_ret_borrowed,
                                     impl_ret_new_ref, impl_ret_untracked)
 from numba.utils import cached_property
@@ -378,8 +377,7 @@ def build_list(context, builder, list_type, items):
     return impl_ret_new_ref(context, builder, list_type, inst.value)
 
 
-@builtin
-@implement(list, types.Kind(types.IterableType))
+@lower_builtin(list, types.IterableType)
 def list_constructor(context, builder, sig, args):
 
     def list_impl(iterable):
@@ -393,8 +391,7 @@ def list_constructor(context, builder, sig, args):
 #-------------------------------------------------------------------------------
 # Various operations
 
-@builtin
-@implement(types.len_type, types.Kind(types.List))
+@lower_builtin(len, types.List)
 def list_len(context, builder, sig, args):
     inst = ListInstance(context, builder, sig.args[0], args[0])
     return inst.size
@@ -407,14 +404,12 @@ def make_listiter_cls(iterator_type):
     """
     return cgutils.create_struct_proxy(iterator_type)
 
-@builtin
-@implement('getiter', types.Kind(types.List))
+@lower_builtin('getiter', types.List)
 def getiter_list(context, builder, sig, args):
     inst = ListIterInstance.from_list(context, builder, sig.return_type, args[0])
     return impl_ret_borrowed(context, builder, sig.return_type, inst.value)
 
-@builtin
-@implement('iternext', types.Kind(types.ListIter))
+@lower_builtin('iternext', types.ListIter)
 @iternext_impl
 def iternext_listiter(context, builder, sig, args, result):
     inst = ListIterInstance(context, builder, sig.args[0], args[0])
@@ -429,8 +424,7 @@ def iternext_listiter(context, builder, sig, args, result):
         inst.index = builder.add(index, context.get_constant(types.intp, 1))
 
 
-@builtin
-@implement('getitem', types.Kind(types.List), types.Kind(types.Integer))
+@lower_builtin('getitem', types.List, types.Integer)
 def getitem_list(context, builder, sig, args):
     inst = ListInstance(context, builder, sig.args[0], args[0])
     index = args[1]
@@ -440,8 +434,7 @@ def getitem_list(context, builder, sig, args):
 
     return impl_ret_borrowed(context, builder, sig.return_type, result)
 
-@builtin
-@implement('setitem', types.Kind(types.List), types.Kind(types.Integer), types.Any)
+@lower_builtin('setitem', types.List, types.Integer, types.Any)
 def setitem_list(context, builder, sig, args):
     inst = ListInstance(context, builder, sig.args[0], args[0])
     index = args[1]
@@ -452,12 +445,11 @@ def setitem_list(context, builder, sig, args):
     return context.get_dummy_value()
 
 
-@builtin
-@implement('getitem', types.Kind(types.List), types.slice3_type)
+@lower_builtin('getitem', types.List, types.SliceType)
 def getslice_list(context, builder, sig, args):
     inst = ListInstance(context, builder, sig.args[0], args[0])
-    slice = slicing.Slice(context, builder, value=args[1])
-    cgutils.guard_invalid_slice(context, builder, slice)
+    slice = slicing.make_slice(context, builder, sig.args[1], args[1])
+    slicing.guard_invalid_slice(context, builder, sig.args[1], slice)
     inst.fix_slice(slice)
 
     # Allocate result and populate it
@@ -476,14 +468,13 @@ def getslice_list(context, builder, sig, args):
 
     return impl_ret_new_ref(context, builder, sig.return_type, result.value)
 
-@builtin
-@implement('setitem', types.Kind(types.List), types.slice3_type, types.Any)
+@lower_builtin('setitem', types.List, types.SliceType, types.Any)
 def setitem_list(context, builder, sig, args):
     dest = ListInstance(context, builder, sig.args[0], args[0])
-    slice = slicing.Slice(context, builder, value=args[1])
     src = ListInstance(context, builder, sig.args[2], args[2])
 
-    cgutils.guard_invalid_slice(context, builder, slice)
+    slice = slicing.make_slice(context, builder, sig.args[1], args[1])
+    slicing.guard_invalid_slice(context, builder, sig.args[1], slice)
     dest.fix_slice(slice)
 
     src_size = src.size
@@ -537,13 +528,12 @@ def setitem_list(context, builder, sig, args):
     return context.get_dummy_value()
 
 
-@builtin
-@implement('delitem', types.Kind(types.List), types.slice3_type)
+@lower_builtin('delitem', types.List, types.SliceType)
 def setitem_list(context, builder, sig, args):
     inst = ListInstance(context, builder, sig.args[0], args[0])
-    slice = slicing.Slice(context, builder, value=args[1])
+    slice = slicing.make_slice(context, builder, sig.args[1], args[1])
 
-    cgutils.guard_invalid_slice(context, builder, slice)
+    slicing.guard_invalid_slice(context, builder, sig.args[1], slice)
     inst.fix_slice(slice)
 
     slice_len = slicing.get_slice_length(builder, slice)
@@ -566,8 +556,7 @@ def setitem_list(context, builder, sig, args):
     return context.get_dummy_value()
 
 
-@builtin
-@implement("in", types.Any, types.Kind(types.List))
+@lower_builtin("in", types.Any, types.List)
 def in_list(context, builder, sig, args):
     def list_contains_impl(value, lst):
         for elem in lst:
@@ -579,8 +568,7 @@ def in_list(context, builder, sig, args):
 
 
 # XXX should there be a specific module for Sequence or collection base classes?
-@builtin
-@implement(bool, types.Kind(types.Sequence))
+@lower_builtin(bool, types.Sequence)
 def sequence_bool(context, builder, sig, args):
     def sequence_bool_impl(seq):
         return len(seq) != 0
@@ -588,8 +576,7 @@ def sequence_bool(context, builder, sig, args):
     return context.compile_internal(builder, sequence_bool_impl, sig, args)
 
 
-@builtin
-@implement("+", types.Kind(types.List), types.Kind(types.List))
+@lower_builtin("+", types.List, types.List)
 def list_add(context, builder, sig, args):
     a = ListInstance(context, builder, sig.args[0], args[0])
     b = ListInstance(context, builder, sig.args[1], args[1])
@@ -611,8 +598,7 @@ def list_add(context, builder, sig, args):
 
     return impl_ret_new_ref(context, builder, sig.return_type, dest.value)
 
-@builtin
-@implement("+=", types.Kind(types.List), types.Kind(types.List))
+@lower_builtin("+=", types.List, types.List)
 def list_add_inplace(context, builder, sig, args):
     assert sig.args[0].dtype == sig.return_type.dtype
     dest = _list_extend_list(context, builder, sig, args)
@@ -620,8 +606,7 @@ def list_add_inplace(context, builder, sig, args):
     return impl_ret_borrowed(context, builder, sig.return_type, dest.value)
 
 
-@builtin
-@implement("*", types.Kind(types.List), types.Kind(types.Integer))
+@lower_builtin("*", types.List, types.Integer)
 def list_mul(context, builder, sig, args):
     src = ListInstance(context, builder, sig.args[0], args[0])
     src_size = src.size
@@ -641,8 +626,7 @@ def list_mul(context, builder, sig, args):
 
     return impl_ret_new_ref(context, builder, sig.return_type, dest.value)
 
-@builtin
-@implement("*=", types.Kind(types.List), types.Kind(types.Integer))
+@lower_builtin("*=", types.List, types.Integer)
 def list_mul_inplace(context, builder, sig, args):
     inst = ListInstance(context, builder, sig.args[0], args[0])
     src_size = inst.size
@@ -665,8 +649,7 @@ def list_mul_inplace(context, builder, sig, args):
 #-------------------------------------------------------------------------------
 # Comparisons
 
-@builtin
-@implement('is', types.Kind(types.List), types.Kind(types.List))
+@lower_builtin('is', types.List, types.List)
 def list_is(context, builder, sig, args):
     a = ListInstance(context, builder, sig.args[0], args[0])
     b = ListInstance(context, builder, sig.args[1], args[1])
@@ -674,8 +657,7 @@ def list_is(context, builder, sig, args):
     mb = builder.ptrtoint(b.meminfo, cgutils.intp_t)
     return builder.icmp_signed('==', ma, mb)
 
-@builtin
-@implement('==', types.Kind(types.List), types.Kind(types.List))
+@lower_builtin('==', types.List, types.List)
 def list_eq(context, builder, sig, args):
     aty, bty = sig.args
     a = ListInstance(context, builder, aty, args[0])
@@ -699,8 +681,7 @@ def list_eq(context, builder, sig, args):
 
     return builder.load(res)
 
-@builtin
-@implement('!=', types.Kind(types.List), types.Kind(types.List))
+@lower_builtin('!=', types.List, types.List)
 def list_ne(context, builder, sig, args):
 
     def list_ne_impl(a, b):
@@ -708,8 +689,7 @@ def list_ne(context, builder, sig, args):
 
     return context.compile_internal(builder, list_ne_impl, sig, args)
 
-@builtin
-@implement('<=', types.Kind(types.List), types.Kind(types.List))
+@lower_builtin('<=', types.List, types.List)
 def list_le(context, builder, sig, args):
 
     def list_le_impl(a, b):
@@ -724,8 +704,7 @@ def list_le(context, builder, sig, args):
 
     return context.compile_internal(builder, list_le_impl, sig, args)
 
-@builtin
-@implement('<', types.Kind(types.List), types.Kind(types.List))
+@lower_builtin('<', types.List, types.List)
 def list_lt(context, builder, sig, args):
 
     def list_lt_impl(a, b):
@@ -740,8 +719,7 @@ def list_lt(context, builder, sig, args):
 
     return context.compile_internal(builder, list_lt_impl, sig, args)
 
-@builtin
-@implement('>=', types.Kind(types.List), types.Kind(types.List))
+@lower_builtin('>=', types.List, types.List)
 def list_ge(context, builder, sig, args):
 
     def list_ge_impl(a, b):
@@ -749,8 +727,7 @@ def list_ge(context, builder, sig, args):
 
     return context.compile_internal(builder, list_ge_impl, sig, args)
 
-@builtin
-@implement('>', types.Kind(types.List), types.Kind(types.List))
+@lower_builtin('>', types.List, types.List)
 def list_gt(context, builder, sig, args):
 
     def list_gt_impl(a, b):
@@ -761,8 +738,7 @@ def list_gt(context, builder, sig, args):
 #-------------------------------------------------------------------------------
 # Methods
 
-@builtin
-@implement("list.append", types.Kind(types.List), types.Any)
+@lower_builtin("list.append", types.List, types.Any)
 def list_append(context, builder, sig, args):
     inst = ListInstance(context, builder, sig.args[0], args[0])
     item = args[1]
@@ -774,24 +750,21 @@ def list_append(context, builder, sig, args):
 
     return context.get_dummy_value()
 
-@builtin
-@implement("list.clear", types.Kind(types.List))
+@lower_builtin("list.clear", types.List)
 def list_clear(context, builder, sig, args):
     inst = ListInstance(context, builder, sig.args[0], args[0])
     inst.resize(context.get_constant(types.intp, 0))
 
     return context.get_dummy_value()
 
-@builtin
-@implement("list.copy", types.Kind(types.List))
+@lower_builtin("list.copy", types.List)
 def list_copy(context, builder, sig, args):
     def list_copy_impl(lst):
         return list(lst)
 
     return context.compile_internal(builder, list_copy_impl, sig, args)
 
-@builtin
-@implement("list.count", types.Kind(types.List), types.Any)
+@lower_builtin("list.count", types.List, types.Any)
 def list_count(context, builder, sig, args):
 
     def list_count_impl(lst, value):
@@ -820,8 +793,7 @@ def _list_extend_list(context, builder, sig, args):
 
     return dest
 
-@builtin
-@implement("list.extend", types.Kind(types.List), types.Kind(types.IterableType))
+@lower_builtin("list.extend", types.List, types.IterableType)
 def list_extend(context, builder, sig, args):
     if isinstance(sig.args[1], types.List):
         # Specialize for list operands, for speed.
@@ -836,8 +808,7 @@ def list_extend(context, builder, sig, args):
 
     return context.compile_internal(builder, list_extend, sig, args)
 
-@builtin
-@implement("list.index", types.Kind(types.List), types.Any)
+@lower_builtin("list.index", types.List, types.Any)
 def list_index(context, builder, sig, args):
 
     def list_index_impl(lst, value):
@@ -849,9 +820,8 @@ def list_index(context, builder, sig, args):
 
     return context.compile_internal(builder, list_index_impl, sig, args)
 
-@builtin
-@implement("list.index", types.Kind(types.List), types.Any,
-           types.Kind(types.Integer))
+@lower_builtin("list.index", types.List, types.Any,
+           types.Integer)
 def list_index(context, builder, sig, args):
 
     def list_index_impl(lst, value, start):
@@ -868,9 +838,8 @@ def list_index(context, builder, sig, args):
 
     return context.compile_internal(builder, list_index_impl, sig, args)
 
-@builtin
-@implement("list.index", types.Kind(types.List), types.Any,
-           types.Kind(types.Integer), types.Kind(types.Integer))
+@lower_builtin("list.index", types.List, types.Any,
+           types.Integer, types.Integer)
 def list_index(context, builder, sig, args):
 
     def list_index_impl(lst, value, start, stop):
@@ -891,8 +860,7 @@ def list_index(context, builder, sig, args):
 
     return context.compile_internal(builder, list_index_impl, sig, args)
 
-@builtin
-@implement("list.insert", types.Kind(types.List), types.Kind(types.Integer),
+@lower_builtin("list.insert", types.List, types.Integer,
            types.Any)
 def list_insert(context, builder, sig, args):
     inst = ListInstance(context, builder, sig.args[0], args[0])
@@ -909,8 +877,7 @@ def list_insert(context, builder, sig, args):
 
     return context.get_dummy_value()
 
-@builtin
-@implement("list.pop", types.Kind(types.List))
+@lower_builtin("list.pop", types.List)
 def list_pop(context, builder, sig, args):
     inst = ListInstance(context, builder, sig.args[0], args[0])
 
@@ -922,8 +889,7 @@ def list_pop(context, builder, sig, args):
     inst.resize(n)
     return res
 
-@builtin
-@implement("list.pop", types.Kind(types.List), types.Kind(types.Integer))
+@lower_builtin("list.pop", types.List, types.Integer)
 def list_pop(context, builder, sig, args):
     inst = ListInstance(context, builder, sig.args[0], args[0])
     idx = inst.fix_index(args[1])
@@ -941,8 +907,7 @@ def list_pop(context, builder, sig, args):
     inst.resize(n)
     return res
 
-@builtin
-@implement("list.remove", types.Kind(types.List), types.Any)
+@lower_builtin("list.remove", types.List, types.Any)
 def list_remove(context, builder, sig, args):
 
     def list_remove_impl(lst, value):
@@ -955,8 +920,7 @@ def list_remove(context, builder, sig, args):
 
     return context.compile_internal(builder, list_remove_impl, sig, args)
 
-@builtin
-@implement("list.reverse", types.Kind(types.List))
+@lower_builtin("list.reverse", types.List)
 def list_reverse(context, builder, sig, args):
 
     def list_reverse_impl(lst):
@@ -990,9 +954,8 @@ def load_sorts():
     g['_sorting_init'] = True
 
 
-@builtin
-@implement("list.sort", types.Kind(types.List))
-@implement("list.sort", types.Kind(types.List), types.Kind(types.Boolean))
+@lower_builtin("list.sort", types.List)
+@lower_builtin("list.sort", types.List, types.Boolean)
 def list_sort(context, builder, sig, args):
     load_sorts()
 
@@ -1008,9 +971,8 @@ def list_sort(context, builder, sig, args):
 
     return context.compile_internal(builder, list_sort_impl, sig, args)
 
-@builtin
-@implement(sorted, types.Kind(types.IterableType))
-@implement(sorted, types.Kind(types.IterableType), types.Kind(types.Boolean))
+@lower_builtin(sorted, types.IterableType)
+@lower_builtin(sorted, types.IterableType, types.Boolean)
 def sorted_impl(context, builder, sig, args):
     if len(args) == 1:
         sig = typing.signature(sig.return_type, *sig.args + (types.boolean,))
@@ -1022,3 +984,13 @@ def sorted_impl(context, builder, sig, args):
         return lst
 
     return context.compile_internal(builder, sorted_impl, sig, args)
+
+
+# -----------------------------------------------------------------------------
+# Implicit casting
+
+@lower_cast(types.List, types.List)
+def list_to_list(context, builder, fromty, toty, val):
+    # Casting from non-reflected to reflected
+    assert fromty.dtype == toty.dtype
+    return val
