@@ -9,7 +9,7 @@ from numba.npyufunc.ufuncbuilder import GUFuncBuilder
 from numba import vectorize, guvectorize
 from numba.npyufunc import PyUFunc_One
 from numba.npyufunc.dufunc import DUFunc as UFuncBuilder
-from numba.tests import support
+from ..support import tag
 
 
 def add(a, b):
@@ -66,6 +66,7 @@ def guerror(a, b, c):
 
 class TestUfuncBuilding(unittest.TestCase):
 
+    @tag('important')
     def test_basic_ufunc(self):
         ufb = UFuncBuilder(add)
         cres = ufb.add("int32(int32, int32)")
@@ -169,6 +170,7 @@ class TestGUfuncBuilding(unittest.TestCase):
         self.assertEqual(ufunc.__name__, "guadd")
         self.assertIn("A generalized addition", ufunc.__doc__)
 
+    @tag('important')
     def test_gufunc_struct(self):
         gufb = GUFuncBuilder(guadd, "(x, y),(x, y)->(x, y)")
         cres = gufb.add("void(complex64[:,:], complex64[:,:], complex64[:,:])")
@@ -225,6 +227,7 @@ class TestVectorizeDecor(unittest.TestCase):
         self.assertTrue(numpy.all(a + a == b))
         self.assertEqual(b.dtype, numpy.dtype('int32'))
 
+    @tag('important')
     def test_vectorize_bool_return(self):
         ufunc = vectorize(['bool_(int32, int32)'])(equals)
         a = numpy.arange(10, dtype='int32')
@@ -232,6 +235,7 @@ class TestVectorizeDecor(unittest.TestCase):
         self.assertTrue(numpy.all(r))
         self.assertEqual(r.dtype, numpy.dtype('bool_'))
 
+    @tag('important')
     def test_vectorize_identity(self):
         sig = 'int32(int32, int32)'
         for identity in self._supported_identities:
@@ -263,6 +267,7 @@ class TestVectorizeDecor(unittest.TestCase):
         ufunc = vectorize(identity=PyUFunc_One, nopython=True)(mul)
         self.assertTrue(numpy.all(ufunc(a,b) == (a * b)))
 
+    @tag('important')
     def test_guvectorize(self):
         ufunc = guvectorize(['(int32[:,:], int32[:,:], int32[:,:])'],
                             "(x,y),(x,y)->(x,y)")(guadd)
@@ -271,6 +276,7 @@ class TestVectorizeDecor(unittest.TestCase):
         self.assertTrue(numpy.all(a + a == b))
         self.assertEqual(b.dtype, numpy.dtype('int32'))
 
+    @tag('important')
     def test_guvectorize_no_output(self):
         ufunc = guvectorize(['(int32[:,:], int32[:,:], int32[:,:])'],
                             "(x,y),(x,y),(x,y)")(guadd)
@@ -293,6 +299,7 @@ class TestVectorizeDecor(unittest.TestCase):
         with self.assertRaises(MyException):
             ufunc(a, a)
 
+    @tag('important')
     def test_guvectorize_identity(self):
         args = (['(int32[:,:], int32[:,:], int32[:,:])'], "(x,y),(x,y)->(x,y)")
         for identity in self._supported_identities:
