@@ -7,7 +7,7 @@ import sys
 import numba.unittest_support as unittest
 from numba.compiler import compile_isolated, Flags
 from numba import jit, typeof, errors, types, utils
-from .support import TestCase
+from .support import TestCase, tag
 
 enable_pyobj_flags = Flags()
 enable_pyobj_flags.set("enable_pyobject")
@@ -202,6 +202,7 @@ class TestBuiltins(TestCase):
         for x in complex_values:
             self.assertPreciseEqual(cfunc(x), pyfunc(x))
 
+    @tag('important')
     def test_abs_npm(self):
         self.test_abs(flags=no_pyobj_flags)
 
@@ -309,6 +310,7 @@ class TestBuiltins(TestCase):
         for x, y in itertools.product(x_operands, y_operands):
             self.assertPreciseEqual(cfunc(x, y), pyfunc(x, y))
 
+    @tag('important')
     def test_complex_npm(self):
         self.test_complex(flags=no_pyobj_flags)
 
@@ -358,6 +360,7 @@ class TestBuiltins(TestCase):
         for x in ['-1.1', '0.0', '1.1']:
             self.assertPreciseEqual(cfunc(x), pyfunc(x))
 
+    @tag('important')
     def test_float_npm(self):
         with self.assertTypingError():
             self.test_float(flags=no_pyobj_flags)
@@ -435,6 +438,7 @@ class TestBuiltins(TestCase):
         for x, y in itertools.product(x_operands, y_operands):
             self.assertPreciseEqual(cfunc(x, y), pyfunc(x, y))
 
+    @tag('important')
     def test_int_npm(self):
         with self.assertTypingError():
             self.test_int(flags=no_pyobj_flags)
@@ -504,6 +508,7 @@ class TestBuiltins(TestCase):
         for x, y in itertools.product(x_operands, y_operands):
             self.assertPreciseEqual(cfunc(x, y), pyfunc(x, y))
 
+    @tag('important')
     def test_max_npm_1(self):
         self.test_max_1(flags=no_pyobj_flags)
 
@@ -531,6 +536,7 @@ class TestBuiltins(TestCase):
         for x, y in itertools.product(x_operands, y_operands):
             self.assertPreciseEqual(cfunc(x, y), pyfunc(x, y))
 
+    @tag('important')
     def test_min_npm_1(self):
         self.test_min_1(flags=no_pyobj_flags)
 
@@ -673,6 +679,7 @@ class TestBuiltins(TestCase):
                         self.assertPreciseEqual(cfunc(-x, n), pyfunc(-x, n),
                                                 prec=prec)
 
+    @tag('important')
     def test_round2_npm(self):
         self.test_round2(flags=no_pyobj_flags)
 
@@ -710,7 +717,7 @@ class TestBuiltins(TestCase):
         check(True, 2)
         check(2.5j, False)
 
-    @unittest.skipIf(utils.IS_PY3, "cmp not available as global is Py3")
+    @unittest.skipIf(utils.IS_PY3, "unichr not available as global is Py3")
     def test_unichr(self, flags=enable_pyobj_flags):
         pyfunc = unichr_usecase
 
@@ -719,7 +726,7 @@ class TestBuiltins(TestCase):
         for x in range(0, 1000, 10):
             self.assertPreciseEqual(cfunc(x), pyfunc(x))
 
-    @unittest.skipIf(utils.IS_PY3, "cmp not available as global is Py3")
+    @unittest.skipIf(utils.IS_PY3, "unichr not available as global is Py3")
     def test_unichr_npm(self):
         with self.assertTypingError():
             self.test_unichr(flags=no_pyobj_flags)
@@ -727,18 +734,21 @@ class TestBuiltins(TestCase):
     def test_zip(self, flags=forceobj_flags):
         self.run_nullary_func(zip_usecase, flags)
 
+    @tag('important')
     def test_zip_npm(self):
         self.test_zip(flags=no_pyobj_flags)
 
     def test_zip_1(self, flags=forceobj_flags):
         self.run_nullary_func(zip_1_usecase, flags)
 
+    @tag('important')
     def test_zip_1_npm(self):
         self.test_zip_1(flags=no_pyobj_flags)
 
     def test_zip_3(self, flags=forceobj_flags):
         self.run_nullary_func(zip_3_usecase, flags)
 
+    @tag('important')
     def test_zip_3_npm(self):
         self.test_zip_3(flags=no_pyobj_flags)
 
@@ -762,6 +772,7 @@ class TestBuiltins(TestCase):
             r = cres.entry_point(x, y)
             self.assertPreciseEqual(r, pow_op_usecase(x, y))
 
+    @tag('important')
     def test_pow_usecase(self):
         args = [
             (2, 3),
@@ -776,6 +787,6 @@ class TestBuiltins(TestCase):
             r = cres.entry_point(x, y)
             self.assertPreciseEqual(r, pow_usecase(x, y))
 
+
 if __name__ == '__main__':
     unittest.main()
-
