@@ -13,9 +13,9 @@ import warnings
 import numpy as np
 
 from numba import unittest_support as unittest
-from numba import utils, vectorize, jit
-from numba.config import NumbaWarning
-from .support import TestCase
+from numba import utils, vectorize, jit, generated_jit, types
+from numba.errors import NumbaWarning
+from .support import TestCase, tag, temp_directory, import_dynamic
 
 
 def dummy(x):
@@ -38,16 +38,40 @@ def star_defaults(x, y=2, *z):
     return x, y, z
 
 
-class TestDispatcher(TestCase):
+def generated_usecase(x, y=5):
+    if isinstance(x, types.Complex):
+        def impl(x, y):
+            return x + y
+    else:
+        def impl(x, y):
+            return x - y
+    return impl
+
+def bad_generated_usecase(x, y=5):
+    if isinstance(x, types.Complex):
+        def impl(x):
+            return x
+    else:
+        def impl(x, y=6):
+            return x - y
+    return impl
+
+
+class BaseTest(TestCase):
+
+    jit_args = dict(nopython=True)
 
     def compile_func(self, pyfunc):
         def check(*args, **kwargs):
             expected = pyfunc(*args, **kwargs)
             result = f(*args, **kwargs)
             self.assertPreciseEqual(result, expected)
-        f = jit(nopython=True)(pyfunc)
+        f = jit(**self.jit_args)(pyfunc)
         return f, check
 
+
+class TestDispatcher(BaseTest):
+
     def test_no_argument(self):
         @jit
         def foo():
@@ -119,81 +143,6 @@ class TestDispatcher(TestCase):
             t.join()
         self.assertFalse(errors)
 
-    def test_named_args(self):
-        """
-        Test passing named arguments to a dispatcher.
-        """
-        f, check = self.compile_func(addsub)
-        check(3, z=10, y=4)
-        check(3, 4, 10)
-        check(x=3, y=4, z=10)
-        # All calls above fall under the same specialization
-        self.assertEqual(len(f.overloads), 1)
-        # Errors
-        with self.assertRaises(TypeError) as cm:
-            f(3, 4, y=6, z=7)
-        self.assertIn("too many arguments: expected 3, got 4",
-                      str(cm.exception))
-        with self.assertRaises(TypeError) as cm:
-            f()
-        self.assertIn("not enough arguments: expected 3, got 0",
-                      str(cm.exception))
-        with self.assertRaises(TypeError) as cm:
-            f(3, 4, y=6)
-        self.assertIn("missing argument 'z'", str(cm.exception))
-
-    def test_default_args(self):
-        """
-        Test omitting arguments with a default value.
-        """
-        f, check = self.compile_func(addsub_defaults)
-        check(3, z=10, y=4)
-        check(3, 4, 10)
-        check(x=3, y=4, z=10)
-        # Now omitting some values
-        check(3, z=10)
-        check(3, 4)
-        check(x=3, y=4)
-        check(3)
-        check(x=3)
-        # All calls above fall under the same specialization
-        self.assertEqual(len(f.overloads), 1)
-        # Errors
-        with self.assertRaises(TypeError) as cm:
-            f(3, 4, y=6, z=7)
-        self.assertIn("too many arguments: expected 3, got 4",
-                      str(cm.exception))
-        with self.assertRaises(TypeError) as cm:
-            f()
-        self.assertIn("not enough arguments: expected at least 1, got 0",
-                      str(cm.exception))
-        with self.assertRaises(TypeError) as cm:
-            f(y=6, z=7)
-        self.assertIn("missing argument 'x'", str(cm.exception))
-
-    def test_star_args(self):
-        """
-        Test a compiled function with starargs in the signature.
-        """
-        f, check = self.compile_func(star_defaults)
-        check(4)
-        check(4, 5)
-        check(4, 5, 6)
-        check(4, 5, 6, 7)
-        check(4, 5, 6, 7, 8)
-        check(x=4)
-        check(x=4, y=5)
-        check(4, y=5)
-        with self.assertRaises(TypeError) as cm:
-            f(4, 5, y=6)
-        self.assertIn("some keyword arguments unexpected", str(cm.exception))
-        with self.assertRaises(TypeError) as cm:
-            f(4, 5, z=6)
-        self.assertIn("some keyword arguments unexpected", str(cm.exception))
-        with self.assertRaises(TypeError) as cm:
-            f(4, x=6)
-        self.assertIn("some keyword arguments unexpected", str(cm.exception))
-
     def test_explicit_signatures(self):
         f = jit("(int64,int64)")(add)
         # Approximate match (unsafe conversion)
@@ -256,6 +205,164 @@ class TestDispatcher(TestCase):
                          "No matching definition for argument type(s) "
                          "complex128, complex128")
 
+    def test_disabled_compilation(self):
+        @jit
+        def foo(a):
+            return a
+
+        foo.compile("(float32,)")
+        foo.disable_compile()
+        with self.assertRaises(RuntimeError) as raises:
+            foo.compile("(int32,)")
+        self.assertEqual(str(raises.exception), "compilation disabled")
+        self.assertEqual(len(foo.signatures), 1)
+
+    def test_disabled_compilation_through_list(self):
+        @jit(["(float32,)", "(int32,)"])
+        def foo(a):
+            return a
+
+        with self.assertRaises(RuntimeError) as raises:
+            foo.compile("(complex64,)")
+        self.assertEqual(str(raises.exception), "compilation disabled")
+        self.assertEqual(len(foo.signatures), 2)
+
+    def test_disabled_compilation_nested_call(self):
+        @jit(["(intp,)"])
+        def foo(a):
+            return a
+
+        @jit
+        def bar():
+            foo(1)
+            foo(np.ones(1))  # no matching definition
+
+        with self.assertRaises(TypeError) as raises:
+            bar()
+        m = "No matching definition for argument type(s) array(float64, 1d, C)"
+        self.assertEqual(str(raises.exception), m)
+
+
+class TestSignatureHandling(BaseTest):
+    """
+    Test support for various parameter passing styles.
+    """
+
+    @tag('important')
+    def test_named_args(self):
+        """
+        Test passing named arguments to a dispatcher.
+        """
+        f, check = self.compile_func(addsub)
+        check(3, z=10, y=4)
+        check(3, 4, 10)
+        check(x=3, y=4, z=10)
+        # All calls above fall under the same specialization
+        self.assertEqual(len(f.overloads), 1)
+        # Errors
+        with self.assertRaises(TypeError) as cm:
+            f(3, 4, y=6, z=7)
+        self.assertIn("too many arguments: expected 3, got 4",
+                      str(cm.exception))
+        with self.assertRaises(TypeError) as cm:
+            f()
+        self.assertIn("not enough arguments: expected 3, got 0",
+                      str(cm.exception))
+        with self.assertRaises(TypeError) as cm:
+            f(3, 4, y=6)
+        self.assertIn("missing argument 'z'", str(cm.exception))
+
+    def test_default_args(self):
+        """
+        Test omitting arguments with a default value.
+        """
+        f, check = self.compile_func(addsub_defaults)
+        check(3, z=10, y=4)
+        check(3, 4, 10)
+        check(x=3, y=4, z=10)
+        # Now omitting some values
+        check(3, z=10)
+        check(3, 4)
+        check(x=3, y=4)
+        check(3)
+        check(x=3)
+        # Errors
+        with self.assertRaises(TypeError) as cm:
+            f(3, 4, y=6, z=7)
+        self.assertIn("too many arguments: expected 3, got 4",
+                      str(cm.exception))
+        with self.assertRaises(TypeError) as cm:
+            f()
+        self.assertIn("not enough arguments: expected at least 1, got 0",
+                      str(cm.exception))
+        with self.assertRaises(TypeError) as cm:
+            f(y=6, z=7)
+        self.assertIn("missing argument 'x'", str(cm.exception))
+
+    def test_star_args(self):
+        """
+        Test a compiled function with starargs in the signature.
+        """
+        f, check = self.compile_func(star_defaults)
+        check(4)
+        check(4, 5)
+        check(4, 5, 6)
+        check(4, 5, 6, 7)
+        check(4, 5, 6, 7, 8)
+        check(x=4)
+        check(x=4, y=5)
+        check(4, y=5)
+        with self.assertRaises(TypeError) as cm:
+            f(4, 5, y=6)
+        self.assertIn("some keyword arguments unexpected", str(cm.exception))
+        with self.assertRaises(TypeError) as cm:
+            f(4, 5, z=6)
+        self.assertIn("some keyword arguments unexpected", str(cm.exception))
+        with self.assertRaises(TypeError) as cm:
+            f(4, x=6)
+        self.assertIn("some keyword arguments unexpected", str(cm.exception))
+
+
+class TestSignatureHandlingObjectMode(TestSignatureHandling):
+    """
+    Sams as TestSignatureHandling, but in object mode.
+    """
+
+    jit_args = dict(forceobj=True)
+
+
+class TestGeneratedDispatcher(TestCase):
+    """
+    Tests for @generated_jit.
+    """
+
+    @tag('important')
+    def test_generated(self):
+        f = generated_jit(nopython=True)(generated_usecase)
+        self.assertEqual(f(8), 8 - 5)
+        self.assertEqual(f(x=8), 8 - 5)
+        self.assertEqual(f(x=8, y=4), 8 - 4)
+        self.assertEqual(f(1j), 5 + 1j)
+        self.assertEqual(f(1j, 42), 42 + 1j)
+        self.assertEqual(f(x=1j, y=7), 7 + 1j)
+
+    def test_signature_errors(self):
+        """
+        Check error reporting when implementation signature doesn't match
+        generating function signature.
+        """
+        f = generated_jit(nopython=True)(bad_generated_usecase)
+        # Mismatching # of arguments
+        with self.assertRaises(TypeError) as raises:
+            f(1j)
+        self.assertIn("should be compatible with signature '(x, y=5)', but has signature '(x)'",
+                      str(raises.exception))
+        # Mismatching defaults
+        with self.assertRaises(TypeError) as raises:
+            f(1)
+        self.assertIn("should be compatible with signature '(x, y=5)', but has signature '(x, y=6)'",
+                      str(raises.exception))
+
 
 class TestDispatcherMethods(TestCase):
 
@@ -293,6 +400,7 @@ class TestDispatcherMethods(TestCase):
         self.assertPreciseEqual(foo(1), 3)
         self.assertPreciseEqual(foo(1.5), 3)
 
+    @tag('important')
     def test_inspect_llvm(self):
         # Create a jited function
         @jit
@@ -383,7 +491,7 @@ class TestCache(TestCase):
     modname = "caching_test_fodder"
 
     def setUp(self):
-        self.tempdir = tempfile.mkdtemp()
+        self.tempdir = temp_directory('test_dispatcher_cache')
         sys.path.insert(0, self.tempdir)
         self.modfile = os.path.join(self.tempdir, self.modname + ".py")
         self.cache_dir = os.path.join(self.tempdir, "__pycache__")
@@ -393,7 +501,6 @@ class TestCache(TestCase):
     def tearDown(self):
         sys.modules.pop(self.modname, None)
         sys.path.remove(self.tempdir)
-        shutil.rmtree(self.tempdir)
 
     def import_module(self):
         # Import a fresh version of the test module.  All jitted functions
@@ -415,7 +522,7 @@ class TestCache(TestCase):
                 except OSError as e:
                     if e.errno != errno.ENOENT:
                         raise
-        mod = __import__(self.modname)
+        mod = import_dynamic(self.modname)
         self.assertEqual(mod.__file__.rstrip('co'), self.modfile)
         return mod
 
@@ -452,6 +559,7 @@ class TestCache(TestCase):
             assert mod.add_objmode_usecase(2, 3) == 6
             assert mod.outer_uncached(3, 2) == 2
             assert mod.outer(3, 2) == 2
+            assert mod.generated_usecase(3, 2) == 1
             packed_rec = mod.record_return(mod.packed_arr, 1)
             assert tuple(packed_rec) == (2, 43.5), packed_rec
             aligned_rec = mod.record_return(mod.aligned_arr, 1)
@@ -480,6 +588,7 @@ class TestCache(TestCase):
         self.assertPreciseEqual(f(2.5, 3), 6.5)
         self.check_cache(6)  # 2 index, 4 data
 
+    @tag('important')
     def test_caching(self):
         self.check_cache(0)
         mod = self.import_module()
@@ -504,6 +613,10 @@ class TestCache(TestCase):
         self.assertPreciseEqual(tuple(rec), (2, 43.5))
         self.check_cache(9)  # 3 index, 6 data
 
+        f = mod.generated_usecase
+        self.assertPreciseEqual(f(3, 2), 1)
+        self.assertPreciseEqual(f(3j, 2), 2 + 3j)
+
         # Check the code runs ok from another process
         self.run_in_separate_process()
 
@@ -605,6 +718,7 @@ class TestCache(TestCase):
         mod.outer(2, 3)
         mod.record_return(mod.packed_arr, 0)
         mod.record_return(mod.aligned_arr, 1)
+        mod.generated_usecase(2, 3)
         mtimes = self.get_cache_mtimes()
 
         mod2 = self.import_module()
