@@ -9,7 +9,7 @@ from numba import unittest_support as unittest
 from numba import njit
 from numba import utils
 from numba.numpy_support import version as numpy_version
-from .support import MemoryLeakMixin, TestCase
+from .support import MemoryLeakMixin, TestCase, tag
 
 
 nrtjit = njit(_nrt=True, nogil=True)
@@ -31,13 +31,11 @@ class BaseTest(TestCase):
                 np.testing.assert_allclose(expected, ret)
 
 
-
 class NrtRefCtTest(MemoryLeakMixin):
     def assert_array_nrt_refct(self, arr, expect):
         self.assertEqual(arr.base.refcount, expect)
 
 
-
 class TestDynArray(NrtRefCtTest, TestCase):
 
     def test_empty_0d(self):
@@ -102,6 +100,7 @@ class TestDynArray(NrtRefCtTest, TestCase):
 
         del got_arr
 
+    @tag('important')
     def test_empty_3d(self):
         def pyfunc(m, n, p):
             arr = np.empty((m, n, p), np.int32)
@@ -127,6 +126,7 @@ class TestDynArray(NrtRefCtTest, TestCase):
 
         del got_arr
 
+    @tag('important')
     def test_empty_2d_sliced(self):
         def pyfunc(m, n, p):
             arr = np.empty((m, n), np.int32)
@@ -151,6 +151,7 @@ class TestDynArray(NrtRefCtTest, TestCase):
 
         del got_arr
 
+    @tag('important')
     def test_return_global_array(self):
         y = np.ones(4, dtype=np.float32)
         initrefct = sys.getrefcount(y)
@@ -176,6 +177,7 @@ class TestDynArray(NrtRefCtTest, TestCase):
         # y is no longer referenced by cfunc
         self.assertEqual(initrefct, sys.getrefcount(y))
 
+    @tag('important')
     def test_return_global_array_sliced(self):
         y = np.ones(4, dtype=np.float32)
 
@@ -206,6 +208,7 @@ class TestDynArray(NrtRefCtTest, TestCase):
         self.assertIs(expected, arr)
         self.assertIs(expected, got)
 
+    @tag('important')
     def test_array_pass_through_sliced(self):
         def pyfunc(y):
             return y[y.size // 2:]
@@ -584,6 +587,7 @@ class TestNdZeros(ConstructorBaseTest, TestCase):
             return pyfunc((m, n))
         self.check_2d(func)
 
+    @tag('important')
     def test_2d_dtype_kwarg(self):
         pyfunc = self.pyfunc
         def func(m, n):
@@ -854,6 +858,73 @@ class TestNdEye(BaseTest):
         self.check_eye_n_m_k(func)
 
 
+class TestNdDiag(TestCase):
+
+    def setUp(self):
+        v = np.array([1, 2, 3])
+        hv = np.array([[1, 2, 3]])
+        vv = np.transpose(hv)
+        self.vectors = [v, hv, vv]
+        a3x4 = np.arange(12).reshape(3, 4)
+        a4x3 = np.arange(12).reshape(4, 3)
+        self.matricies = [a3x4, a4x3]
+        def func(q):
+            return np.diag(q)
+        self.py = func
+        self.jit = nrtjit(func)
+
+        def func_kwarg(q, k=0):
+            return np.diag(q, k=k)
+        self.py_kw = func_kwarg
+        self.jit_kw = nrtjit(func_kwarg)
+
+    def check_diag(self, pyfunc, nrtfunc, *args, **kwargs):
+        expected = pyfunc(*args, **kwargs)
+        computed = nrtfunc(*args, **kwargs)
+        self.assertEqual(computed.size, expected.size)
+        self.assertEqual(computed.dtype, expected.dtype)
+        # NOTE: stride not tested as np returns a RO view, nb returns new data
+        np.testing.assert_equal(expected, computed)
+
+    # create a diag matrix from a vector
+    def test_diag_vect_create(self):
+        for d in self.vectors:
+            self.check_diag(self.py, self.jit, d)
+
+    # create a diag matrix from a vector at a given offset
+    def test_diag_vect_create_kwarg(self):
+        for k in range(-10, 10):
+            for d in self.vectors:
+                self.check_diag(self.py_kw, self.jit_kw, d, k=k)
+
+    # extract the diagonal
+    def test_diag_extract(self):
+        for d in self.matricies:
+            self.check_diag(self.py, self.jit, d)
+
+    # extract a diagonal at a given offset
+    def test_diag_extract_kwarg(self):
+        for k in range(-4, 4):
+            for d in self.matricies:
+                self.check_diag(self.py_kw, self.jit_kw, d, k=k)
+
+    # check error handling
+    def test_error_handling(self):
+        from numba.errors import TypingError
+        d = np.array([[[1.]]])
+        cfunc = nrtjit(self.py)
+
+        # missing arg
+        with self.assertRaises(TypeError):
+            cfunc()
+
+        # > 2d
+        with self.assertRaises(TypingError):
+            cfunc(d)
+        with self.assertRaises(TypingError):
+            dfunc = nrtjit(self.py_kw)
+            dfunc(d, k=3)
+
 class TestNdArange(BaseTest):
 
     def test_linspace_2(self):
@@ -922,6 +993,81 @@ class TestNpyEmptyKeyword(TestCase):
             cfunc()
 
 
+class TestNpArray(MemoryLeakMixin, BaseTest):
+
+    def test_0d(self):
+        def pyfunc(arg):
+            return np.array(arg)
+
+        cfunc = nrtjit(pyfunc)
+        got = cfunc(42)
+        self.assertPreciseEqual(got, np.array(42, dtype=np.intp))
+        got = cfunc(2.5)
+        self.assertPreciseEqual(got, np.array(2.5))
+
+    def test_0d_with_dtype(self):
+        def pyfunc(arg):
+            return np.array(arg, dtype=np.int16)
+
+        self.check_outputs(pyfunc, [(42,), (3.5,)])
+
+    def test_1d(self):
+        def pyfunc(arg):
+            return np.array(arg)
+
+        cfunc = nrtjit(pyfunc)
+        # A list
+        got = cfunc([2, 3, 42])
+        self.assertPreciseEqual(got, np.intp([2, 3, 42]))
+        # A heterogenous tuple
+        got = cfunc((1.0, 2.5j, 42))
+        self.assertPreciseEqual(got, np.array([1.0, 2.5j, 42]))
+        # An empty tuple
+        got = cfunc(())
+        self.assertPreciseEqual(got, np.float64(()))
+
+    def test_1d_with_dtype(self):
+        def pyfunc(arg):
+            return np.array(arg, dtype=np.float32)
+
+        self.check_outputs(pyfunc,
+                           [([2, 42],),
+                            ([3.5, 1.0],),
+                            ((1, 3.5, 42),),
+                            ((),),
+                            ])
+
+    @tag('important')
+    def test_2d(self):
+        def pyfunc(arg):
+            return np.array(arg)
+
+        cfunc = nrtjit(pyfunc)
+        # A list of tuples
+        got = cfunc([(1, 2), (3, 4)])
+        self.assertPreciseEqual(got, np.intp([[1, 2], [3, 4]]))
+        got = cfunc([(1, 2.5), (3, 4.5)])
+        self.assertPreciseEqual(got, np.float64([[1, 2.5], [3, 4.5]]))
+        # A tuple of lists
+        got = cfunc(([1, 2], [3, 4]))
+        self.assertPreciseEqual(got, np.intp([[1, 2], [3, 4]]))
+        got = cfunc(([1, 2], [3.5, 4.5]))
+        self.assertPreciseEqual(got, np.float64([[1, 2], [3.5, 4.5]]))
+        # A tuple of tuples
+        got = cfunc(((1.5, 2), (3.5, 4.5)))
+        self.assertPreciseEqual(got, np.float64([[1.5, 2], [3.5, 4.5]]))
+        got = cfunc(((), ()))
+        self.assertPreciseEqual(got, np.float64(((), ())))
+
+    def test_2d_with_dtype(self):
+        def pyfunc(arg):
+            return np.array(arg, dtype=np.int32)
+
+        cfunc = nrtjit(pyfunc)
+        got = cfunc([(1, 2.5), (3, 4.5)])
+        self.assertPreciseEqual(got, np.int32([[1, 2], [3, 4]]))
+
+
 def benchmark_refct_speed():
     def pyfunc(x, y, t):
         """Swap array x and y for t number of times
