@@ -3,10 +3,10 @@ from __future__ import print_function
 import sys
 import numpy as np
 
+import numba.unittest_support as unittest
 from numba.compiler import compile_isolated, Flags
 from numba import jit, types
-from numba.tests.support import TestCase, MemoryLeakMixin
-import numba.unittest_support as unittest
+from .support import TestCase, MemoryLeakMixin, tag
 from numba import testing
 
 enable_pyobj_flags = Flags()
@@ -119,6 +119,7 @@ class TestGenerators(MemoryLeakMixin, TestCase):
         cgen = cr.entry_point(8)
         self.check_generator(pygen, cgen)
 
+    @tag('important')
     def test_gen1(self):
         self.check_gen1()
 
@@ -132,6 +133,7 @@ class TestGenerators(MemoryLeakMixin, TestCase):
         cgen = cr.entry_point(8)
         self.check_generator(pygen, cgen)
 
+    @tag('important')
     def test_gen2(self):
         self.check_gen2()
 
@@ -145,6 +147,7 @@ class TestGenerators(MemoryLeakMixin, TestCase):
         cgen = cr.entry_point(8)
         self.check_generator(pygen, cgen)
 
+    @tag('important')
     def test_gen3(self):
         self.check_gen3()
 
@@ -158,6 +161,7 @@ class TestGenerators(MemoryLeakMixin, TestCase):
         cgen = cr.entry_point(5, 6, 7)
         self.check_generator(pygen, cgen)
 
+    @tag('important')
     def test_gen4(self):
         self.check_gen4()
 
@@ -186,6 +190,7 @@ class TestGenerators(MemoryLeakMixin, TestCase):
             l.append(next(cgen))
         self.assertEqual(l, [14] * 3)
 
+    @tag('important')
     def test_gen6(self):
         self.check_gen6()
 
@@ -201,6 +206,7 @@ class TestGenerators(MemoryLeakMixin, TestCase):
         cgen = cr.entry_point(arr)
         self.check_generator(pygen, cgen)
 
+    @tag('important')
     def test_gen7(self):
         self.check_gen7()
 
@@ -221,6 +227,7 @@ class TestGenerators(MemoryLeakMixin, TestCase):
     def test_consume_gen2(self):
         self.check_consume_generator(gen2)
 
+    @tag('important')
     def test_consume_gen3(self):
         self.check_consume_generator(gen3)
 
