@@ -9,7 +9,7 @@ import numpy as np
 from numba import unittest_support as unittest
 from numba.compiler import compile_isolated, Flags, utils
 from numba import types
-from .support import TestCase, CompilationCache
+from .support import TestCase, CompilationCache, tag
 
 PY27_AND_ABOVE = utils.PYVERSION > (2, 6)
 
@@ -220,6 +220,7 @@ class TestMathLib(TestCase):
         x_values = [-2, -1, -2, 2, 1, 2, .1, .2]
         self.run_unary(pyfunc, x_types, x_values, flags)
 
+    @tag('important')
     def test_sin_npm(self):
         self.test_sin(flags=no_pyobj_flags)
 
@@ -233,6 +234,7 @@ class TestMathLib(TestCase):
         x_values = [-2, -1, -2, 2, 1, 2, .1, .2]
         self.run_unary(pyfunc, x_types, x_values, flags)
 
+    @tag('important')
     @unittest.skipIf(sys.platform == 'win32',
                      "not exactly equal on win32 (issue #597)")
     def test_cos_npm(self):
@@ -246,6 +248,7 @@ class TestMathLib(TestCase):
         x_values = [-2, -1, -2, 2, 1, 2, .1, .2]
         self.run_unary(pyfunc, x_types, x_values, flags)
 
+    @tag('important')
     def test_tan_npm(self):
         self.test_tan(flags=no_pyobj_flags)
 
@@ -257,6 +260,7 @@ class TestMathLib(TestCase):
         x_values = [2, 1, 2, 2, 1, 2, .1, .2]
         self.run_unary(pyfunc, x_types, x_values, flags)
 
+    @tag('important')
     def test_sqrt_npm(self):
         self.test_sqrt(flags=no_pyobj_flags)
 
@@ -282,6 +286,7 @@ class TestMathLib(TestCase):
         x_values = [-2, -1, -2, 2, 1, 2, .1, .2]
         self.run_unary(pyfunc, x_types, x_values, flags)
 
+    @tag('important')
     def test_exp_npm(self):
         self.test_exp(flags=no_pyobj_flags)
 
@@ -294,6 +299,7 @@ class TestMathLib(TestCase):
         x_values = [-2, -1, -2, 2, 1, 2, .1, .2]
         self.run_unary(pyfunc, x_types, x_values, flags)
 
+    @tag('important')
     @unittest.skipIf(not PY27_AND_ABOVE, "Only support for 2.7+")
     def test_expm1_npm(self):
         self.test_expm1(flags=no_pyobj_flags)
@@ -306,6 +312,7 @@ class TestMathLib(TestCase):
         x_values = [1, 10, 100, 1000, 100000, 1000000, 0.1, 1.1]
         self.run_unary(pyfunc, x_types, x_values, flags)
 
+    @tag('important')
     def test_log_npm(self):
         self.test_log(flags=no_pyobj_flags)
 
@@ -317,6 +324,7 @@ class TestMathLib(TestCase):
         x_values = [1, 10, 100, 1000, 100000, 1000000, 0.1, 1.1]
         self.run_unary(pyfunc, x_types, x_values, flags)
 
+    @tag('important')
     def test_log1p_npm(self):
         self.test_log1p(flags=no_pyobj_flags)
 
@@ -328,6 +336,7 @@ class TestMathLib(TestCase):
         x_values = [1, 10, 100, 1000, 100000, 1000000, 0.1, 1.1]
         self.run_unary(pyfunc, x_types, x_values, flags)
 
+    @tag('important')
     def test_log10_npm(self):
         self.test_log10(flags=no_pyobj_flags)
 
@@ -339,6 +348,7 @@ class TestMathLib(TestCase):
         x_values = [1, 1, 1, 1, 1, 1, 1., 1.]
         self.run_unary(pyfunc, x_types, x_values, flags)
 
+    @tag('important')
     def test_asin_npm(self):
         self.test_asin(flags=no_pyobj_flags)
 
@@ -350,6 +360,7 @@ class TestMathLib(TestCase):
         x_values = [1, 1, 1, 1, 1, 1, 1., 1.]
         self.run_unary(pyfunc, x_types, x_values, flags)
 
+    @tag('important')
     def test_acos_npm(self):
         self.test_acos(flags=no_pyobj_flags)
 
@@ -361,6 +372,7 @@ class TestMathLib(TestCase):
         x_values = [-2, -1, -2, 2, 1, 2, .1, .2]
         self.run_unary(pyfunc, x_types, x_values, flags)
 
+    @tag('important')
     def test_atan_npm(self):
         self.test_atan(flags=no_pyobj_flags)
 
@@ -373,6 +385,7 @@ class TestMathLib(TestCase):
         y_values = [x * 2 for x in x_values]
         self.run_binary(pyfunc, x_types, x_values, y_values, flags)
 
+    @tag('important')
     def test_atan2_npm(self):
         self.test_atan2(flags=no_pyobj_flags)
 
@@ -382,8 +395,9 @@ class TestMathLib(TestCase):
                    types.uint16, types.uint32, types.uint64,
                    types.float32, types.float64]
         x_values = [1, 1, 1, 1, 1, 1, 1., 1.]
-        self.run_unary(pyfunc, x_types, x_values, flags)
+        self.run_unary(pyfunc, x_types, x_values, flags, prec='double')
 
+    @tag('important')
     def test_asinh_npm(self):
         self.test_asinh(flags=no_pyobj_flags)
 
@@ -395,6 +409,7 @@ class TestMathLib(TestCase):
         x_values = [1, 1, 1, 1, 1, 1, 1., 1.]
         self.run_unary(pyfunc, x_types, x_values, flags)
 
+    @tag('important')
     def test_acosh_npm(self):
         self.test_acosh(flags=no_pyobj_flags)
 
@@ -404,8 +419,9 @@ class TestMathLib(TestCase):
                    types.uint16, types.uint32, types.uint64,
                    types.float32, types.float64]
         x_values = [0, 0, 0, 0, 0, 0, 0.1, 0.1]
-        self.run_unary(pyfunc, x_types, x_values, flags)
+        self.run_unary(pyfunc, x_types, x_values, flags, prec='double')
 
+    @tag('important')
     def test_atanh_npm(self):
         self.test_atanh(flags=no_pyobj_flags)
 
@@ -417,6 +433,7 @@ class TestMathLib(TestCase):
         x_values = [1, 1, 1, 1, 1, 1, 1., 1.]
         self.run_unary(pyfunc, x_types, x_values, flags)
 
+    @tag('important')
     def test_sinh_npm(self):
         self.test_sinh(flags=no_pyobj_flags)
 
@@ -428,6 +445,7 @@ class TestMathLib(TestCase):
         x_values = [1, 1, 1, 1, 1, 1, 1., 1.]
         self.run_unary(pyfunc, x_types, x_values, flags)
 
+    @tag('important')
     def test_cosh_npm(self):
         self.test_cosh(flags=no_pyobj_flags)
 
@@ -439,6 +457,7 @@ class TestMathLib(TestCase):
         x_values = [0, 0, 0, 0, 0, 0, 0.1, 0.1]
         self.run_unary(pyfunc, x_types, x_values, flags)
 
+    @tag('important')
     def test_tanh_npm(self):
         self.test_tanh(flags=no_pyobj_flags)
 
@@ -450,6 +469,7 @@ class TestMathLib(TestCase):
         x_values = [0, 0, 0, 0, 0, 0, 0.1, 1.9]
         self.run_unary(pyfunc, x_types, x_values, flags)
 
+    @tag('important')
     def test_floor_npm(self):
         self.test_floor(flags=no_pyobj_flags)
 
@@ -461,6 +481,7 @@ class TestMathLib(TestCase):
         x_values = [0, 0, 0, 0, 0, 0, 0.1, 1.9]
         self.run_unary(pyfunc, x_types, x_values, flags)
 
+    @tag('important')
     def test_ceil_npm(self):
         self.test_ceil(flags=no_pyobj_flags)
 
@@ -472,18 +493,21 @@ class TestMathLib(TestCase):
         x_values = [0, 0, 0, 0, 0, 0, 0.1, 1.9]
         self.run_unary(pyfunc, x_types, x_values, flags)
 
+    @tag('important')
     def test_trunc_npm(self):
         self.test_trunc(flags=no_pyobj_flags)
 
     def test_isnan(self):
         self.check_predicate_func(isnan, flags=enable_pyobj_flags)
 
+    @tag('important')
     def test_isnan_npm(self):
         self.check_predicate_func(isnan, flags=no_pyobj_flags)
 
     def test_isinf(self):
         self.check_predicate_func(isinf, flags=enable_pyobj_flags)
 
+    @tag('important')
     def test_isinf_npm(self):
         self.check_predicate_func(isinf, flags=no_pyobj_flags)
 
@@ -506,6 +530,7 @@ class TestMathLib(TestCase):
         prec = 'single' if sys.platform == 'win32' else 'exact'
         self.run_binary(pyfunc, x_types, x_values, y_values, prec=prec)
 
+    @tag('important')
     def test_hypot_npm(self):
         self.test_hypot(flags=no_pyobj_flags)
 
@@ -536,7 +561,8 @@ class TestMathLib(TestCase):
         pyfunc = erf
         x_values = [1., 1., -1., -0.0, 0.0, 0.5, 5, float('inf')]
         x_types = [types.float32, types.float64] * (len(x_values) // 2)
-        self.run_unary(pyfunc, x_types, x_values, flags)
+        self.run_unary(pyfunc, x_types, x_values, flags,
+                       prec='double', ulps=2)
 
     @unittest.skipIf(not PY27_AND_ABOVE, "Only support for 2.7+")
     def test_erf_npm(self):
@@ -547,7 +573,8 @@ class TestMathLib(TestCase):
         pyfunc = erfc
         x_values = [1., 1., -1., -0.0, 0.0, 0.5, 5, float('inf')]
         x_types = [types.float32, types.float64] * (len(x_values) // 2)
-        self.run_unary(pyfunc, x_types, x_values, flags)
+        self.run_unary(pyfunc, x_types, x_values, flags,
+                       prec='double', ulps=4)
 
     @unittest.skipIf(not PY27_AND_ABOVE, "Only support for 2.7+")
     def test_erfc_npm(self):
@@ -561,7 +588,8 @@ class TestMathLib(TestCase):
         self.run_unary(pyfunc, x_types, x_values, flags, prec='double', ulps=3)
         x_values = [-0.1, 0.1, 2.5, 10.1, 50., float('inf')]
         x_types = [types.float64] * len(x_values)
-        self.run_unary(pyfunc, x_types, x_values, flags, prec='double', ulps=3)
+        self.run_unary(pyfunc, x_types, x_values, flags,
+                       prec='double', ulps=8)
 
     @unittest.skipIf(not PY27_AND_ABOVE, "Only support for 2.7+")
     def test_gamma_npm(self):
@@ -587,6 +615,7 @@ class TestMathLib(TestCase):
         y_values = [x * 2 for x in x_values]
         self.run_binary(pyfunc, x_types, x_values, y_values, flags)
 
+    @tag('important')
     def test_pow_npm(self):
         self.test_pow(flags=no_pyobj_flags)
 
@@ -599,6 +628,7 @@ class TestMathLib(TestCase):
             *itertools.product(value_types, values, values)))
         self.run_binary(pyfunc, x_types, x_values, y_values, flags)
 
+    @tag('important')
     def test_copysign_npm(self):
         self.test_copysign(flags=no_pyobj_flags)
 
