@@ -7,14 +7,14 @@ import warnings
 
 import numpy as np
 
+import numba.unittest_support as unittest
 from numba.compiler import compile_isolated, Flags
 from numba import types, typeinfer, utils, errors
 from numba.config import PYVERSION
-from .support import TestCase
+from .support import TestCase, tag
 from .true_div_usecase import truediv_usecase, itruediv_usecase
 from .matmul_usecase import (matmul_usecase, imatmul_usecase, DumbMatrix,
                              needs_matmul, needs_blas)
-import numba.unittest_support as unittest
 
 Noflags = Flags()
 
@@ -350,6 +350,11 @@ class FunctionalOperatorImpl(object):
 
 
 class TestOperators(TestCase):
+    """
+    Test standard Python operators on scalars.
+
+    NOTE: operators on array are generally tested in test_ufuncs.
+    """
 
     op = LiteralOperatorImpl
 
@@ -423,30 +428,6 @@ class TestOperators(TestCase):
                                  "mismatch with %r (%r, %r)"
                                  % (typ, x, y))
 
-    def run_test_array_compare(self, pyfunc, flags=force_pyobj_flags,
-                               ordered=True):
-        ops = np.array(self.compare_scalar_operands)
-        types_list = self.compare_types
-        if not ordered:
-            types_list = types_list + self.compare_unordered_types
-        for typ in types_list:
-            array_type = types.Array(typ, 1, 'C')
-            cr = compile_isolated(pyfunc, (array_type, array_type),
-                                  flags=flags)
-            cfunc = cr.entry_point
-            with warnings.catch_warnings():
-                warnings.simplefilter('ignore', np.ComplexWarning)
-                arr = typ.cast_python_value(ops)
-            for i in range(len(arr)):
-                x = arr
-                y = np.concatenate((arr[i:], arr[:i]))
-                expected = pyfunc(x, y)
-                got = cfunc(x, y)
-                # Array ops => array result
-                self.assertEqual(got.dtype, expected.dtype)
-                self.assertTrue(np.all(got == expected),
-                                "mismatch with %r (%r, %r): %r != %r"
-                                % (typ, x, y, got, expected))
 
     #
     # Comparison operators
@@ -461,80 +442,45 @@ class TestOperators(TestCase):
     def test_lt_scalar(self, flags=force_pyobj_flags):
         self.run_test_scalar_compare(self.op.lt_usecase, flags)
 
+    @tag('important')
     def test_lt_scalar_npm(self):
         self.test_lt_scalar(flags=Noflags)
 
     def test_le_scalar(self, flags=force_pyobj_flags):
         self.run_test_scalar_compare(self.op.le_usecase, flags)
 
+    @tag('important')
     def test_le_scalar_npm(self):
         self.test_le_scalar(flags=Noflags)
 
     def test_gt_scalar(self, flags=force_pyobj_flags):
         self.run_test_scalar_compare(self.op.gt_usecase, flags)
 
+    @tag('important')
     def test_gt_scalar_npm(self):
         self.test_gt_scalar(flags=Noflags)
 
     def test_ge_scalar(self, flags=force_pyobj_flags):
         self.run_test_scalar_compare(self.op.ge_usecase, flags)
 
+    @tag('important')
     def test_ge_scalar_npm(self):
         self.test_ge_scalar(flags=Noflags)
 
     def test_eq_scalar(self, flags=force_pyobj_flags):
         self.run_test_scalar_compare(self.op.eq_usecase, flags, ordered=False)
 
+    @tag('important')
     def test_eq_scalar_npm(self):
         self.test_eq_scalar(flags=Noflags)
 
     def test_ne_scalar(self, flags=force_pyobj_flags):
         self.run_test_scalar_compare(self.op.ne_usecase, flags, ordered=False)
 
+    @tag('important')
     def test_ne_scalar_npm(self):
         self.test_ne_scalar(flags=Noflags)
 
-    def test_eq_array(self, flags=force_pyobj_flags):
-        self.run_test_array_compare(self.op.eq_usecase, flags, ordered=False)
-
-    def test_eq_array_npm(self):
-        with self.assertTypingError():
-            self.test_eq_array(flags=Noflags)
-
-    def test_ne_array(self, flags=force_pyobj_flags):
-        self.run_test_array_compare(self.op.ne_usecase, flags, ordered=False)
-
-    def test_ne_array_npm(self):
-        with self.assertTypingError():
-            self.test_ne_array(flags=Noflags)
-
-    def test_lt_array(self, flags=force_pyobj_flags):
-        self.run_test_array_compare(self.op.lt_usecase, flags)
-
-    def test_lt_array_npm(self):
-        with self.assertTypingError():
-            self.test_lt_array(flags=Noflags)
-
-    def test_le_array(self, flags=force_pyobj_flags):
-        self.run_test_array_compare(self.op.le_usecase, flags)
-
-    def test_le_array_npm(self):
-        with self.assertTypingError():
-            self.test_le_array(flags=Noflags)
-
-    def test_gt_array(self, flags=force_pyobj_flags):
-        self.run_test_array_compare(self.op.gt_usecase, flags)
-
-    def test_gt_array_npm(self):
-        with self.assertTypingError():
-            self.test_gt_array(flags=Noflags)
-
-    def test_ge_array(self, flags=force_pyobj_flags):
-        self.run_test_array_compare(self.op.ge_usecase, flags)
-
-    def test_ge_array_npm(self):
-        with self.assertTypingError():
-            self.test_ge_array(flags=Noflags)
 
     #
     # Arithmetic operators
@@ -580,42 +526,6 @@ class TestOperators(TestCase):
         self.run_test_floats(pyfunc, x_operands, y_operands, types_list,
                              flags=flags)
 
-    def run_binop_array_ints(self, pyfunc, flags=force_pyobj_flags):
-        array = np.arange(-10, 10, dtype=np.int32)
-
-        x_operands = [array[1:]]
-        y_operands = [array[:-1]]
-
-        arraytype = types.Array(types.int32, 1, 'C')
-        types_list = [(arraytype, arraytype)]
-
-        self.run_test_ints(pyfunc, x_operands, y_operands, types_list,
-                           flags=flags)
-
-    def run_binop_array_floats(self, pyfunc, flags=force_pyobj_flags):
-        array = np.arange(-1, 1, 0.1, dtype=np.float32)
-
-        x_operands = [array]
-        y_operands = [array]
-
-        arraytype = types.Array(types.float32, 1, 'C')
-        types_list = [(arraytype, arraytype)]
-
-        self.run_test_floats(pyfunc, x_operands, y_operands, types_list,
-                             flags=flags)
-
-    def run_binop_array_complex(self, pyfunc, flags=force_pyobj_flags):
-        array = np.arange(-1, 1, 0.1, dtype=np.complex64) * (1.5 + 0.8j)
-
-        x_operands = [array]
-        y_operands = [array]
-
-        arraytype = types.Array(types.complex64, 1, 'C')
-        types_list = [(arraytype, arraytype)]
-
-        self.run_test_floats(pyfunc, x_operands, y_operands, types_list,
-                             flags=flags)
-
     def generate_binop_tests(ns, usecases, tp_runners, npm_array=False):
         for usecase in usecases:
             for tp_name, runner_name in tp_runners.items():
@@ -640,6 +550,10 @@ class TestOperators(TestCase):
                         test_meth = inner
 
                     test_meth.__name__ = test_name
+
+                    if nopython:
+                        test_meth = tag('important')(test_meth)
+
                     ns[test_name] = test_meth
 
 
@@ -648,19 +562,13 @@ class TestOperators(TestCase):
                          {'ints': 'run_binop_ints',
                           'floats': 'run_binop_floats',
                           'complex': 'run_binop_complex',
-                          'ints_array': 'run_binop_array_ints',
-                          'floats_array': 'run_binop_array_floats',
-                          'complex_array': 'run_binop_array_complex',
                           })
 
-    # NOTE: truediv doesn't handle int arrays in Numpy 1.10+
     generate_binop_tests(locals(),
                          ('div', 'idiv', 'truediv', 'itruediv'),
                          {'ints': 'run_binop_ints',
                           'floats': 'run_binop_floats',
                           'complex': 'run_binop_complex',
-                          'floats_array': 'run_binop_array_floats',
-                          'complex_array': 'run_binop_array_complex',
                           })
 
     # NOTE: floordiv and mod unsupported for complex numbers
@@ -668,8 +576,6 @@ class TestOperators(TestCase):
                          ('floordiv', 'ifloordiv', 'mod', 'imod'),
                          {'ints': 'run_binop_ints',
                           'floats': 'run_binop_floats',
-                          'ints_array': 'run_binop_array_ints',
-                          'floats_array': 'run_binop_array_floats',
                           })
 
     def check_div_errors(self, usecase_name, msg, flags=force_pyobj_flags,
@@ -755,53 +661,11 @@ class TestOperators(TestCase):
         self.run_test_floats(pyfunc, x_operands, y_operands, types_list,
                              flags=flags)
 
-    def run_pow_ints_array(self, pyfunc, flags=force_pyobj_flags):
-        array = np.arange(-10, 10, dtype=np.int32)
-
-        x_operands = [array]
-        y_operands = [array]
-
-        arraytype = types.Array(types.int32, 1, 'C')
-        types_list = [(arraytype, arraytype)]
-
-        self.run_test_ints(pyfunc, x_operands, y_operands, types_list,
-                           flags=flags)
-
-    def run_pow_floats_array(self, pyfunc, flags=force_pyobj_flags):
-        # NOTE
-        # If x is finite negative and y is finite but not an integer,
-        # it causes a domain error
-        array = np.arange(0.1, 1, 0.1, dtype=np.float32)
-
-        x_operands = [array]
-        y_operands = [array]
-
-        arraytype = types.Array(types.float32, 1, 'C')
-        types_list = [(arraytype, arraytype)]
-
-        self.run_test_ints(pyfunc, x_operands, y_operands, types_list,
-                           flags=flags)
-
-
-        x_array = np.arange(-1, 0.1, 0.1, dtype=np.float32)
-        y_array = np.arange(len(x_array), dtype=np.float32)
-
-        x_operands = [x_array]
-        y_operands = [y_array]
-
-        arraytype = types.Array(types.float32, 1, 'C')
-        types_list = [(arraytype, arraytype)]
-
-        self.run_test_ints(pyfunc, x_operands, y_operands, types_list,
-                           flags=flags)
-
     # XXX power operator is unsupported on complex numbers (see issue #488)
     generate_binop_tests(locals(),
                          ('pow', 'ipow'),
                          {'ints': 'run_pow_ints',
                           'floats': 'run_pow_floats',
-                          'ints_array': 'run_pow_ints_array',
-                          'floats_array': 'run_pow_floats_array',
                           })
 
     def test_add_complex(self, flags=force_pyobj_flags):
@@ -816,6 +680,7 @@ class TestOperators(TestCase):
         self.run_test_floats(pyfunc, x_operands, y_operands, types_list,
                              flags=flags)
 
+    @tag('important')
     def test_add_complex_npm(self):
         self.test_add_complex(flags=Noflags)
 
@@ -831,6 +696,7 @@ class TestOperators(TestCase):
         self.run_test_floats(pyfunc, x_operands, y_operands, types_list,
                              flags=flags)
 
+    @tag('important')
     def test_sub_complex_npm(self):
         self.test_sub_complex(flags=Noflags)
 
@@ -846,6 +712,7 @@ class TestOperators(TestCase):
         self.run_test_floats(pyfunc, x_operands, y_operands, types_list,
                              flags=flags)
 
+    @tag('important')
     def test_mul_complex_npm(self):
         self.test_mul_complex(flags=Noflags)
 
@@ -861,6 +728,7 @@ class TestOperators(TestCase):
         self.run_test_floats(pyfunc, x_operands, y_operands, types_list,
                              flags=flags)
 
+    @tag('important')
     def test_div_complex_npm(self):
         self.test_div_complex(flags=Noflags)
 
@@ -876,6 +744,7 @@ class TestOperators(TestCase):
         self.run_test_floats(pyfunc, x_operands, y_operands, types_list,
                              flags=flags)
 
+    @tag('important')
     def test_truediv_complex_npm(self):
         self.test_truediv_complex(flags=Noflags)
 
@@ -884,6 +753,7 @@ class TestOperators(TestCase):
         with self.assertTypingError():
             cres = compile_isolated(pyfunc, (types.complex64, types.complex64))
 
+    @tag('important')
     def test_mod_complex_npm(self):
         self.test_mod_complex(flags=Noflags)
 
@@ -930,10 +800,12 @@ class TestOperators(TestCase):
         self.assertIsNot(got, a)
         self.assertIsNot(got, b)
 
+    @tag('important')
     @needs_matmul
     def test_matmul_npm(self):
         self.check_matmul_npm(self.op.matmul_usecase)
 
+    @tag('important')
     @needs_matmul
     def test_imatmul_npm(self):
         with self.assertTypingError() as raises:
@@ -976,22 +848,9 @@ class TestOperators(TestCase):
         self.run_test_ints(pyfunc, x_operands, y_operands, types_list,
                            flags=flags)
 
-    def run_bitshift_array(self, pyfunc, flags=force_pyobj_flags):
-        array = np.arange(0, 10, dtype=np.int32)
-
-        x_operands = [array[:-1]]
-        y_operands = [array[1:]]
-
-        arraytype = types.Array(types.int32, 1, 'C')
-        types_list = [(arraytype, arraytype)]
-
-        self.run_test_ints(pyfunc, x_operands, y_operands, types_list,
-                           flags=flags)
-
     generate_binop_tests(locals(),
                          ('bitshift_left', 'bitshift_ileft'),
                          {'ints': 'run_bitshift_left',
-                          'ints_array': 'run_bitshift_array',
                           })
 
     def run_bitshift_right(self, pyfunc, flags=force_pyobj_flags):
@@ -1030,7 +889,6 @@ class TestOperators(TestCase):
     generate_binop_tests(locals(),
                          ('bitshift_right', 'bitshift_iright'),
                          {'ints': 'run_bitshift_right',
-                          'ints_array': 'run_bitshift_array',
                           })
 
     def run_logical(self, pyfunc, flags=force_pyobj_flags):
@@ -1066,28 +924,11 @@ class TestOperators(TestCase):
         self.run_test_ints(pyfunc, x_operands, y_operands, types_list,
                            flags=flags)
 
-    def run_logical_array(self, pyfunc, flags=force_pyobj_flags):
-        dtype = np.int32
-        array = np.concatenate([
-            np.array([-(2**31), 2**31 - 1], dtype=dtype),
-            np.arange(-10, 10, dtype=dtype),
-            ])
-
-        x_operands = [array[:-1]]
-        y_operands = [array[1:]]
-
-        arraytype = types.Array(types.int32, 1, 'C')
-        types_list = [(arraytype, arraytype)]
-
-        self.run_test_ints(pyfunc, x_operands, y_operands, types_list,
-                           flags=flags)
-
     generate_binop_tests(locals(),
                          ('bitwise_and', 'bitwise_iand',
                           'bitwise_or', 'bitwise_ior',
                           'bitwise_xor', 'bitwise_ixor'),
                          {'ints': 'run_logical',
-                          'ints_array': 'run_logical_array',
                           })
 
     #
@@ -1132,6 +973,7 @@ class TestOperators(TestCase):
         self.run_test_ints(pyfunc, x_operands, y_operands, types_list,
                            flags=flags)
 
+    @tag('important')
     def test_bitwise_not_npm(self):
         self.test_bitwise_not(flags=Noflags)
 
@@ -1151,6 +993,7 @@ class TestOperators(TestCase):
         for val in values:
             self.assertEqual(pyfunc(val), cfunc(val))
 
+    @tag('important')
     def test_not_npm(self):
         pyfunc = self.op.not_usecase
         # test native mode
@@ -1176,6 +1019,7 @@ class TestOperators(TestCase):
 
     # XXX test_negate should check for negative and positive zeros and infinites
 
+    @tag('important')
     def test_negate_npm(self):
         pyfunc = self.op.negate_usecase
         # test native mode
@@ -1347,12 +1191,15 @@ class TestMixedInts(TestCase):
         self.run_binary(pyfunc, self.get_control_unsigned(opname),
                         samples, self.unsigned_pairs, expected_type)
 
+    @tag('important')
     def test_add(self):
         self.run_arith_binop(self.op.add_usecase, 'add', self.int_samples)
 
+    @tag('important')
     def test_sub(self):
         self.run_arith_binop(self.op.sub_usecase, 'sub', self.int_samples)
 
+    @tag('important')
     def test_mul(self):
         self.run_arith_binop(self.op.mul_usecase, 'mul', self.int_samples)
 
