@@ -16,11 +16,11 @@ except ImportError:
     setuptools = None
 
 from numba import unittest_support as unittest
-from numba.pycc import find_shared_ending, find_pyext_ending, main
+from numba.pycc import main
 from numba.pycc.decorators import clear_export_registry
-from .support import TestCase
+from numba.pycc.platform import find_shared_ending, find_pyext_ending
+from .support import TestCase, tag, import_dynamic, temp_directory
 
-from numba.tests.support import static_temp_directory
 
 base_path = os.path.dirname(os.path.abspath(__file__))
 
@@ -37,7 +37,7 @@ def unset_macosx_deployment_target():
 class BasePYCCTest(TestCase):
 
     def setUp(self):
-        self.tmpdir = static_temp_directory('test_pycc')
+        self.tmpdir = temp_directory('test_pycc')
         # Make sure temporary files and directories created by
         # distutils don't clutter the top-level /tmp
         tempfile.tempdir = self.tmpdir
@@ -53,7 +53,7 @@ class BasePYCCTest(TestCase):
     def check_c_ext(self, extdir, name):
         sys.path.append(extdir)
         try:
-            lib = __import__(name)
+            lib = import_dynamic(name)
             yield lib
         finally:
             sys.path.remove(extdir)
@@ -176,7 +176,7 @@ class TestCC(BasePYCCTest):
         f = self._test_module.cc.output_file
         self.assertFalse(os.path.exists(f), f)
         self.assertTrue(os.path.basename(f).startswith('pycc_test_simple.'), f)
-        if sys.platform == 'linux':
+        if sys.platform.startswith('linux'):
             self.assertTrue(f.endswith('.so'), f)
             if sys.version_info >= (3,):
                 self.assertIn('.cpython', f)
@@ -193,6 +193,7 @@ class TestCC(BasePYCCTest):
             with self.assertRaises(ZeroDivisionError):
                 lib.div(1, 0)
 
+    @tag('important')
     def test_compile_helperlib(self):
         with self.check_cc_compiled(self._test_module.cc_helperlib) as lib:
             res = lib.power(2, 7)
@@ -218,6 +219,7 @@ class TestCC(BasePYCCTest):
                 """ % {'expected': expected}
             self.check_cc_compiled_in_subprocess(lib, code)
 
+    @tag('important')
     def test_compile_nrt(self):
         with self.check_cc_compiled(self._test_module.cc_nrt) as lib:
             # Sanity check
@@ -239,14 +241,11 @@ class TestDistutilsSupport(TestCase):
     def setUp(self):
         # Copy the test project into a temp directory to avoid
         # keeping any build leftovers in the source tree
-        self.tmpdir = tempfile.mkdtemp(prefix='test_pycc_distutils-')
+        self.tmpdir = temp_directory('test_pycc_distutils')
         source_dir = os.path.join(base_path, 'pycc_distutils_usecase')
         self.usecase_dir = os.path.join(self.tmpdir, 'work')
         shutil.copytree(source_dir, self.usecase_dir)
 
-    def tearDown(self):
-        shutil.rmtree(self.tmpdir)
-
     def check_setup_py(self, setup_py_file):
         # Compute PYTHONPATH to ensure the child processes see this Numba
         import numba
