@@ -12,7 +12,8 @@ import numpy as np
 import numba.unittest_support as unittest
 from numba import jit, _helperlib, types
 from numba.compiler import compile_isolated
-from .support import TestCase, compile_function
+from numba.targets.randomimpl import random_init
+from .support import TestCase, compile_function, tag
 
 
 # State size of the Mersenne Twister
@@ -187,6 +188,10 @@ class TestRandom(TestCase):
     # iterative algorithms with sensitive exit conditions.
     # Therefore we stick to hardcoded integers for seed values below.
 
+    def setUp(self):
+        # Make sure the PRNG is initialized before we set the seed
+        random_init()
+
     def _follow_cpython(self, ptr, seed=2):
         r = random.Random(seed)
         _copy_py_state(r, ptr)
@@ -211,9 +216,11 @@ class TestRandom(TestCase):
             for j in range(N + 10):
                 self.assertPreciseEqual(randomfunc(), r.uniform(0.0, 1.0))
 
+    @tag('important')
     def test_random_random(self):
         self._check_random_seed(random_seed, random_random)
 
+    @tag('important')
     def test_numpy_random(self):
         self._check_random_seed(numpy_seed, numpy_random)
         # Test aliases
@@ -249,6 +256,7 @@ class TestRandom(TestCase):
         self.assertRaises(OverflowError, func, 9999999)
         self.assertRaises(OverflowError, func, -1)
 
+    @tag('important')
     def test_random_getrandbits(self):
         self._check_getrandbits(jit_unary("random.getrandbits"), py_state_ptr)
 
@@ -281,6 +289,7 @@ class TestRandom(TestCase):
         if func0 is not None:
             self._check_dist(func0, r.normal, [()])
 
+    @tag('important')
     def test_random_gauss(self):
         self._check_gauss(jit_binary("random.gauss"), None, None, py_state_ptr)
 
@@ -290,16 +299,19 @@ class TestRandom(TestCase):
         self._check_gauss(jit_binary("random.normalvariate"), None, None,
                           py_state_ptr)
 
+    @tag('important')
     def test_numpy_normal(self):
         self._check_gauss(jit_binary("np.random.normal"),
                           jit_unary("np.random.normal"),
                           jit_nullary("np.random.normal"),
                           np_state_ptr)
 
+    @tag('important')
     def test_numpy_standard_normal(self):
         self._check_gauss(None, None, jit_nullary("np.random.standard_normal"),
                           np_state_ptr)
 
+    @tag('important')
     def test_numpy_randn(self):
         self._check_gauss(None, None, jit_nullary("np.random.randn"),
                           np_state_ptr)
@@ -362,6 +374,7 @@ class TestRandom(TestCase):
             self.assertRaises(ValueError, func3, 5, 7, -1)
             self.assertRaises(ValueError, func3, 7, 5, 1)
 
+    @tag('important')
     def test_random_randrange(self):
         for tp, max_width in [(types.int64, 2**63), (types.int32, 2**31)]:
             cr1 = compile_isolated(random_randrange1, (tp,))
@@ -370,6 +383,7 @@ class TestRandom(TestCase):
             self._check_randrange(cr1.entry_point, cr2.entry_point,
                                   cr3.entry_point, py_state_ptr, max_width)
 
+    @tag('important')
     def test_numpy_randint(self):
         for tp, max_width in [(types.int64, 2**63), (types.int32, 2**31)]:
             cr1 = compile_isolated(numpy_randint1, (tp,))
@@ -397,6 +411,7 @@ class TestRandom(TestCase):
         self.assertRaises(ValueError, func, 5, 4)
         self.assertRaises(ValueError, func, 5, 2)
 
+    @tag('important')
     def test_random_randint(self):
         for tp, max_width in [(types.int64, 2**63), (types.int32, 2**31)]:
             cr = compile_isolated(random_randint, (tp, tp))
@@ -411,9 +426,11 @@ class TestRandom(TestCase):
         self._check_dist(func, r.uniform,
                          [(1.5, 1e6), (-2.5, 1e3), (1.5, -2.5)])
 
+    @tag('important')
     def test_random_uniform(self):
         self._check_uniform(jit_binary("random.uniform"), py_state_ptr)
 
+    @tag('important')
     def test_numpy_uniform(self):
         self._check_uniform(jit_binary("np.random.uniform"), np_state_ptr)
 
@@ -586,6 +603,7 @@ class TestRandom(TestCase):
         self._check_weibullvariate(None, jit_unary("np.random.weibull"),
                                    np_state_ptr)
 
+    @tag('important')
     def test_numpy_binomial(self):
         # We follow Numpy's algorithm up to n*p == 30
         self._follow_numpy(np_state_ptr, 0)
@@ -594,6 +612,7 @@ class TestRandom(TestCase):
         self.assertRaises(ValueError, binomial, 10, -0.1)
         self.assertRaises(ValueError, binomial, 10, 1.1)
 
+    @tag('important')
     def test_numpy_chisquare(self):
         chisquare = jit_unary("np.random.chisquare")
         r = self._follow_cpython(np_state_ptr)
@@ -661,6 +680,7 @@ class TestRandom(TestCase):
                          [(0.0,), (-1.5,)])
         self._check_dist(jit_nullary("np.random.laplace"), r.laplace, [()])
 
+    @tag('important')
     def test_numpy_logistic(self):
         r = self._follow_numpy(np_state_ptr)
         self._check_dist(jit_binary("np.random.logistic"), r.logistic,
@@ -713,6 +733,7 @@ class TestRandom(TestCase):
         self.assertRaises(ValueError, negbin, 10, -0.1)
         self.assertRaises(ValueError, negbin, 10, 1.1)
 
+    @tag('important')
     def test_numpy_power(self):
         r = self._follow_numpy(np_state_ptr)
         power = jit_unary("np.random.power")
@@ -794,9 +815,11 @@ class TestRandom(TestCase):
             with self.assertTypingError():
                 func(memoryview(b"xyz"))
 
+    @tag('important')
     def test_random_shuffle(self):
         self._check_shuffle(jit_unary("random.shuffle"), py_state_ptr)
 
+    @tag('important')
     def test_numpy_shuffle(self):
         self._check_shuffle(jit_unary("np.random.shuffle"), np_state_ptr)
 
@@ -835,4 +858,3 @@ class TestRandom(TestCase):
 
 if __name__ == "__main__":
     unittest.main()
-
