@@ -1,5 +1,6 @@
 from __future__ import print_function, absolute_import, division
 
+import contextlib
 import pickle
 import subprocess
 import sys
@@ -7,7 +8,7 @@ import sys
 from numba import unittest_support as unittest
 from numba.errors import TypingError
 from numba.targets import registry
-from .support import TestCase
+from .support import TestCase, tag
 from .serialize_usecases import *
 
 
@@ -17,10 +18,18 @@ class TestDispatcherPickling(TestCase):
         for proto in range(pickle.HIGHEST_PROTOCOL + 1):
             meth(proto, *args, **kwargs)
 
+    @contextlib.contextmanager
     def simulate_fresh_target(self):
         dispatcher_cls = registry.dispatcher_registry['cpu']
+        old_descr = dispatcher_cls.targetdescr
         # Simulate fresh targetdescr
         dispatcher_cls.targetdescr = type(dispatcher_cls.targetdescr)()
+        try:
+            yield
+        finally:
+            # Be sure to reinstantiate old descriptor, otherwise other
+            # objects may be out of sync.
+            dispatcher_cls.targetdescr = old_descr
 
     def check_call(self, proto, func, expected_result, args):
         def check_result(func):
@@ -33,21 +42,24 @@ class TestDispatcherPickling(TestCase):
         # Control
         check_result(func)
         pickled = pickle.dumps(func, proto)
-        self.simulate_fresh_target()
-        new_func = pickle.loads(pickled)
-        check_result(new_func)
+        with self.simulate_fresh_target():
+            new_func = pickle.loads(pickled)
+            check_result(new_func)
 
+    @tag('important')
     def test_call_with_sig(self):
         self.run_with_protocols(self.check_call, add_with_sig, 5, (1, 4))
         # Compilation has been disabled => float inputs will be coerced to int
         self.run_with_protocols(self.check_call, add_with_sig, 5, (1.2, 4.2))
 
+    @tag('important')
     def test_call_without_sig(self):
         self.run_with_protocols(self.check_call, add_without_sig, 5, (1, 4))
         self.run_with_protocols(self.check_call, add_without_sig, 5.5, (1.2, 4.3))
         # Object mode is enabled
         self.run_with_protocols(self.check_call, add_without_sig, "abc", ("a", "bc"))
 
+    @tag('important')
     def test_call_nopython(self):
         self.run_with_protocols(self.check_call, add_nopython, 5.5, (1.2, 4.3))
         # Object mode is disabled
@@ -97,6 +109,13 @@ class TestDispatcherPickling(TestCase):
         self.run_with_protocols(self.check_call, get_renamed_module,
                                 expected, (0.0,))
 
+    def test_call_generated(self):
+        self.run_with_protocols(self.check_call, generated_add,
+                                46, (1, 2))
+        self.run_with_protocols(self.check_call, generated_add,
+                                1j + 7, (1j, 2))
+
+    @tag('important')
     def test_other_process(self):
         """
         Check that reconstructing doesn't depend on resources already
