@@ -17,7 +17,7 @@ from numba.npdatetime import NPDATETIME_SUPPORTED
 from numba.special import typeof
 from numba._dispatcher import compute_fingerprint
 
-from .support import TestCase
+from .support import TestCase, tag
 from .test_numpy_support import ValueTypingTestBase
 from .ctypes_usecases import *
 
@@ -54,6 +54,7 @@ class TestTypeof(ValueTypingTestBase, TestCase):
     Test typeof() and, implicitly, typing.Context.get_argument_type().
     """
 
+    @tag('important')
     def test_number_values(self):
         """
         Test special.typeof() with scalar number values.
@@ -67,18 +68,21 @@ class TestTypeof(ValueTypingTestBase, TestCase):
         self.assertEqual(typeof(2**63 - 1), types.int64)
         self.assertEqual(typeof(-2**63), types.int64)
 
+    @tag('important')
     def test_datetime_values(self):
         """
         Test special.typeof() with np.timedelta64 values.
         """
         self.check_datetime_values(typeof)
 
+    @tag('important')
     def test_timedelta_values(self):
         """
         Test special.typeof() with np.timedelta64 values.
         """
         self.check_timedelta_values(typeof)
 
+    @tag('important')
     def test_array_values(self):
         """
         Test special.typeof() with ndarray values.
@@ -103,6 +107,7 @@ class TestTypeof(ValueTypingTestBase, TestCase):
         a4.flags.writeable = False
         check(a4, 0, 'C', False, True)
 
+    @tag('important')
     def test_structured_arrays(self):
         def check(arr, dtype, ndim, layout, aligned):
             ty = typeof(arr)
@@ -152,10 +157,12 @@ class TestTypeof(ValueTypingTestBase, TestCase):
         self.assertEqual(ty, types.ByteArray(types.uint8, 1, "C"))
         self.assertTrue(ty.mutable)
 
+    @tag('important')
     def test_none(self):
         ty = typeof(None)
         self.assertEqual(ty, types.none)
 
+    @tag('important')
     def test_ellipsis(self):
         ty = typeof(Ellipsis)
         self.assertEqual(ty, types.ellipsis)
@@ -164,6 +171,7 @@ class TestTypeof(ValueTypingTestBase, TestCase):
         ty = typeof("abc")
         self.assertEqual(ty, types.string)
 
+    @tag('important')
     def test_slices(self):
         for args in [(1,), (1, 2), (1, 2, 1), (1, 2, None)]:
             v = slice(*args)
@@ -172,6 +180,7 @@ class TestTypeof(ValueTypingTestBase, TestCase):
             v = slice(*args)
             self.assertIs(typeof(v), types.slice3_type)
 
+    @tag('important')
     def test_tuples(self):
         v = (1, 2)
         self.assertEqual(typeof(v), types.UniTuple(types.intp, 2))
@@ -181,10 +190,12 @@ class TestTypeof(ValueTypingTestBase, TestCase):
                                       types.Tuple((types.float64, types.intp))))
                          )
 
+    @tag('important')
     def test_lists(self):
         v = [1.0] * 100
         self.assertEqual(typeof(v), types.List(types.float64, reflected=True))
 
+    @tag('important')
     def test_namedtuple(self):
         v = Point(1, 2)
         tp_point = typeof(v)
@@ -200,6 +211,7 @@ class TestTypeof(ValueTypingTestBase, TestCase):
         self.assertNotEqual(tp_rect, tp_point)
         self.assertNotEqual(tp_rect, types.UniTuple(tp_rect.dtype, tp_rect.count))
 
+    @tag('important')
     def test_dtype(self):
         dtype = np.dtype('int64')
         self.assertEqual(typeof(dtype), types.DType(types.int64))
@@ -208,6 +220,7 @@ class TestTypeof(ValueTypingTestBase, TestCase):
         rec_ty = numpy_support.from_struct_dtype(dtype)
         self.assertEqual(typeof(dtype), types.DType(rec_ty))
 
+    @tag('important')
     def test_ctypes(self):
         ty_cos = typeof(c_cos)
         ty_sin = typeof(c_sin)
@@ -218,20 +231,22 @@ class TestTypeof(ValueTypingTestBase, TestCase):
         self.assertNotEqual(ty_cos.get_pointer(c_cos),
                             ty_sin.get_pointer(c_sin))
 
+    @tag('important')
     @unittest.skipUnless(cffi_support.SUPPORTED, "CFFI not supported")
     def test_cffi(self):
-        from .cffi_usecases import cffi_cos, cffi_sin
-        ty_cffi_cos = typeof(cffi_cos)
-        ty_cffi_sin = typeof(cffi_sin)
+        from . import cffi_usecases as mod
+        mod.init()
+        ty_cffi_cos = typeof(mod.cffi_cos)
+        ty_cffi_sin = typeof(mod.cffi_sin)
         self.assertIsInstance(ty_cffi_cos, types.ExternalFunctionPointer)
         self.assertEqual(ty_cffi_cos.sig.args, (types.float64,))
         self.assertEqual(ty_cffi_cos.sig.return_type, types.float64)
         self.assertEqual(ty_cffi_cos, ty_cffi_sin)
         ty_ctypes_cos = typeof(c_cos)
         self.assertNotEqual(ty_cffi_cos, ty_ctypes_cos)
-        self.assertNotEqual(ty_cffi_cos.get_pointer(cffi_cos),
-                            ty_cffi_sin.get_pointer(cffi_sin))
-        self.assertEqual(ty_cffi_cos.get_pointer(cffi_cos),
+        self.assertNotEqual(ty_cffi_cos.get_pointer(mod.cffi_cos),
+                            ty_cffi_sin.get_pointer(mod.cffi_sin))
+        self.assertEqual(ty_cffi_cos.get_pointer(mod.cffi_cos),
                          ty_ctypes_cos.get_pointer(c_cos))
 
     def test_custom(self):
