@@ -18,7 +18,7 @@ from numba import abstracttypes, types, typing
 from numba import jit, numpy_support
 from numba import unittest_support as unittest
 from numba.npdatetime import NPDATETIME_SUPPORTED
-from .support import TestCase
+from .support import TestCase, tag
 
 
 def gen(x):
@@ -31,6 +31,7 @@ class Dummy(object):
 
 class TestTypes(TestCase):
 
+    @tag('important')
     def test_equality(self):
         self.assertEqual(types.int32, types.int32)
         self.assertEqual(types.uint32, types.uint32)
@@ -151,6 +152,7 @@ class TestTypes(TestCase):
         self.assertTrue(b != c)
         self.assertTrue(a != z)
 
+    @tag('important')
     def test_interning(self):
         # Test interning and lifetime of dynamic types.
         a = types.Dummy('xyzzyx')
@@ -186,6 +188,7 @@ class TestTypes(TestCase):
         gc.collect()
         self.assertEqual(len(cache), cache_len)
 
+    @tag('important')
     def test_array_notation(self):
         def check(arrty, scalar, ndim, layout):
             self.assertIs(arrty.dtype, scalar)
@@ -198,6 +201,7 @@ class TestTypes(TestCase):
         check(scalar[:,::1], scalar, 2, 'C')
         check(scalar[::1,:], scalar, 2, 'F')
 
+    @tag('important')
     def test_call_notation(self):
         # Function call signature
         i = types.int32
