@@ -16,7 +16,7 @@ from numba import vectorize
 from numba.config import PYVERSION
 from numba.errors import LoweringError, TypingError
 from .support import (TestCase, CompilationCache, skip_on_numpy_16,
-                      is_on_numpy_16, MemoryLeakMixin)
+                      is_on_numpy_16, MemoryLeakMixin, tag)
 
 from numba.typing.npydecl import supported_ufuncs, all_ufuncs
 
@@ -161,7 +161,7 @@ class BaseUFuncTest(MemoryLeakMixin):
 
 class TestUFuncs(BaseUFuncTest, TestCase):
 
-    def unary_ufunc_test(self, ufunc, flags=enable_pyobj_flags,
+    def unary_ufunc_test(self, ufunc, flags=no_pyobj_flags,
                          skip_inputs=[], additional_inputs=[],
                          int_output_type=None, float_output_type=None,
                          kinds='ifc'):
@@ -243,7 +243,7 @@ class TestUFuncs(BaseUFuncTest, TestCase):
                                                expected.dtype, expected)
                         self.fail(msg)
 
-    def binary_ufunc_test(self, ufunc, flags=enable_pyobj_flags,
+    def binary_ufunc_test(self, ufunc, flags=no_pyobj_flags,
                          skip_inputs=[], additional_inputs=[],
                          int_output_type=None, float_output_type=None,
                          kinds='ifc'):
@@ -285,13 +285,13 @@ class TestUFuncs(BaseUFuncTest, TestCase):
             pyfunc(input_operand, input_operand, expected)
             np.testing.assert_array_almost_equal(expected, result)
 
-    def unary_int_ufunc_test(self, name=None, flags=enable_pyobj_flags):
+    def unary_int_ufunc_test(self, name=None, flags=no_pyobj_flags):
         self.unary_ufunc_test(name, flags=flags,
             skip_inputs=[types.float32, types.float64,
                 types.Array(types.float32, 1, 'C'),
                 types.Array(types.float64, 1, 'C')])
 
-    def binary_int_ufunc_test(self, name=None, flags=enable_pyobj_flags):
+    def binary_int_ufunc_test(self, name=None, flags=no_pyobj_flags):
         self.binary_ufunc_test(name, flags=flags,
             skip_inputs=[types.float32, types.float64,
                 types.Array(types.float32, 1, 'C'),
@@ -300,25 +300,21 @@ class TestUFuncs(BaseUFuncTest, TestCase):
 
     ############################################################################
     # Math operations
-    def test_add_ufunc(self, flags=enable_pyobj_flags):
+
+    @tag('important')
+    def test_add_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.add, flags=flags)
 
-    def test_add_ufunc_npm(self):
-        self.test_add_ufunc(flags=no_pyobj_flags)
-
-    def test_subtract_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_subtract_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.subtract, flags=flags)
 
-    def test_subtract_ufunc_npm(self):
-        self.test_subtract_ufunc(flags=no_pyobj_flags)
-
-    def test_multiply_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_multiply_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.multiply, flags=flags)
 
-    def test_multiply_ufunc_npm(self):
-        self.test_multiply_ufunc(flags=no_pyobj_flags)
-
-    def test_divide_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_divide_ufunc(self, flags=no_pyobj_flags):
         # Bear in mind that in python3 divide IS true_divide
         # so the out type for int types will be a double
         int_out_type = None
@@ -327,68 +323,46 @@ class TestUFuncs(BaseUFuncTest, TestCase):
 
         self.binary_ufunc_test(np.divide, flags=flags, int_output_type=int_out_type)
 
-    def test_divide_ufunc_npm(self):
-        self.test_divide_ufunc(flags=no_pyobj_flags)
-
     def test_logaddexp_ufunc(self):
         self.binary_ufunc_test(np.logaddexp, kinds='f')
 
-    def test_logaddexp_ufunc_npm(self):
-        self.binary_ufunc_test(np.logaddexp, flags=no_pyobj_flags, kinds='f')
-
     def test_logaddexp2_ufunc(self):
         self.binary_ufunc_test(np.logaddexp2, kinds='f')
 
-    def test_logaddexp2_ufunc_npm(self):
-        self.binary_ufunc_test(np.logaddexp2, flags=no_pyobj_flags, kinds='f')
-
-    def test_true_divide_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_true_divide_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.true_divide, flags=flags, int_output_type=types.float64)
 
-    def test_true_divide_ufunc_npm(self):
-        self.test_true_divide_ufunc(flags=no_pyobj_flags)
-
+    @tag('important')
     def test_floor_divide_ufunc(self):
         self.binary_ufunc_test(np.floor_divide)
 
-    def test_floor_divide_ufunc_npm(self):
-        self.binary_ufunc_test(np.floor_divide, flags=no_pyobj_flags)
-
-    def test_negative_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_negative_ufunc(self, flags=no_pyobj_flags):
         # NumPy ufunc has bug with uint32 as input and int64 as output,
         # so skip uint32 input.
         self.unary_ufunc_test(np.negative, int_output_type=types.int64,
                               skip_inputs=[types.Array(types.uint32, 1, 'C'), types.uint32],
                               flags=flags)
 
-    def test_negative_ufunc_npm(self):
-        self.test_negative_ufunc(flags=no_pyobj_flags)
-
-    def test_power_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_power_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.power, flags=flags)
 
-    def test_power_ufunc_npm(self):
-        self.test_power_ufunc(flags=no_pyobj_flags)
-
-    def test_remainder_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_remainder_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.remainder, flags=flags)
 
-    def test_remainder_ufunc_npm(self):
-        self.test_remainder_ufunc(flags=no_pyobj_flags)
-
-    def test_mod_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_mod_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.mod, flags=flags)
 
-    def test_mod_ufunc_npm(self):
-        self.test_mod_ufunc(flags=no_pyobj_flags)
-
-    def test_fmod_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_fmod_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.fmod, flags=flags)
 
-    def test_fmod_ufunc_npm(self):
-        self.test_fmod_ufunc(flags=no_pyobj_flags)
-
-    def test_abs_ufunc(self, flags=enable_pyobj_flags, ufunc=np.abs):
+    @tag('important')
+    def test_abs_ufunc(self, flags=no_pyobj_flags, ufunc=np.abs):
         self.unary_ufunc_test(ufunc, flags=flags,
             additional_inputs = [
                 (np.uint32(np.iinfo(np.uint32).max), types.uint32),
@@ -397,94 +371,64 @@ class TestUFuncs(BaseUFuncTest, TestCase):
                 (np.float64(np.finfo(np.float64).min), types.float64)
                 ])
 
-    def test_abs_ufunc_npm(self):
-        self.test_abs_ufunc(flags=no_pyobj_flags)
-
-    def test_absolute_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_absolute_ufunc(self, flags=no_pyobj_flags):
         self.test_abs_ufunc(flags=flags, ufunc=np.absolute)
 
-    def test_absolute_ufunc_npm(self):
-        self.test_absolute_ufunc(flags=no_pyobj_flags)
-
-    def test_fabs_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_fabs_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.fabs, flags=flags, kinds='f')
 
-    def test_fabs_ufunc_npm(self):
-        self.test_fabs_ufunc(flags=no_pyobj_flags)
-
-    def test_rint_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_rint_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.rint, flags=flags, kinds='cf')
 
-    def test_rint_ufunc_npm(self):
-        self.test_rint_ufunc(flags=no_pyobj_flags)
-
-    def test_sign_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_sign_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.sign, flags=flags)
 
-    def test_sign_ufunc_npm(self):
-        self.test_sign_ufunc(flags=no_pyobj_flags)
-
-    def test_conj_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_conj_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.conj, flags=flags)
 
-    def test_conj_ufunc_npm(self):
-        self.test_conj_ufunc(flags=no_pyobj_flags)
-
-    def test_exp_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_exp_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.exp, flags=flags, kinds='cf')
 
-    def test_exp_ufunc_npm(self):
-        self.test_exp_ufunc(flags=no_pyobj_flags)
-
-    def test_exp2_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_exp2_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.exp2, flags=flags, kinds='cf')
 
-    def test_exp2_ufunc_npm(self):
-        self.test_exp2_ufunc(flags=no_pyobj_flags)
-
-    def test_log_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_log_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.log, flags=flags, kinds='cf')
 
-    def test_log_ufunc_npm(self):
-        self.test_log_ufunc(flags=no_pyobj_flags)
-
-    def test_log2_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_log2_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.log2, flags=flags, kinds='cf')
 
-    def test_log2_ufunc_npm(self):
-        self.test_log2_ufunc(flags=no_pyobj_flags)
-
-    def test_log10_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_log10_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.log10, flags=flags, kinds='cf')
 
-    def test_log10_ufunc_npm(self):
-        self.test_log10_ufunc(flags=no_pyobj_flags)
-
-    def test_expm1_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_expm1_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.expm1, flags=flags, kinds='cf')
 
-    def test_expm1_ufunc_npm(self):
-        self.test_expm1_ufunc(flags=no_pyobj_flags)
-
-    def test_log1p_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_log1p_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.log1p, flags=flags, kinds='cf')
 
-    def test_log1p_ufunc_npm(self):
-        self.test_log1p_ufunc(flags=no_pyobj_flags)
-
-    def test_sqrt_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_sqrt_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.sqrt, flags=flags, kinds='cf')
 
-    def test_sqrt_ufunc_npm(self):
-        self.test_sqrt_ufunc(flags=no_pyobj_flags)
-
-    def test_square_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_square_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.square, flags=flags)
 
-    def test_square_ufunc_npm(self):
-        self.test_square_ufunc(flags=no_pyobj_flags)
-
-    def test_reciprocal_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_reciprocal_ufunc(self, flags=no_pyobj_flags):
         # reciprocal for integers doesn't make much sense and is problematic
         # in the case of division by zero, as an inf will overflow float to
         # int conversions, which is undefined behavior.
@@ -494,98 +438,60 @@ class TestUFuncs(BaseUFuncTest, TestCase):
                    types.Array(types.int64, 1, 'C'), types.int64]
         self.unary_ufunc_test(np.reciprocal, skip_inputs=to_skip, flags=flags)
 
-    def test_reciprocal_ufunc_npm(self):
-        self.test_reciprocal_ufunc(flags=no_pyobj_flags)
-
-    def test_conjugate_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_conjugate_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.conjugate, flags=flags)
 
-    def test_conjugate_ufunc_npm(self):
-        self.test_conjugate_ufunc(flags=no_pyobj_flags)
-
 
     ############################################################################
     # Trigonometric Functions
 
-    def test_sin_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_sin_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.sin, flags=flags, kinds='cf')
 
-    def test_sin_ufunc_npm(self):
-        self.test_sin_ufunc(flags=no_pyobj_flags)
-
-    def test_cos_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_cos_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.cos, flags=flags, kinds='cf')
 
-    def test_cos_ufunc_npm(self):
-        self.test_cos_ufunc(flags=no_pyobj_flags)
-
-    def test_tan_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_tan_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.tan, flags=flags, kinds='cf')
 
-    def test_tan_ufunc_npm(self):
-        self.test_tan_ufunc(flags=no_pyobj_flags)
-
-    def test_arcsin_ufunc(self, flags=enable_pyobj_flags):
+    def test_arcsin_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.arcsin, flags=flags, kinds='cf')
 
-    def test_arcsin_ufunc_npm(self):
-        self.test_arcsin_ufunc(flags=no_pyobj_flags)
-
-    def test_arccos_ufunc(self, flags=enable_pyobj_flags):
+    def test_arccos_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.arccos, flags=flags, kinds='cf')
 
-    def test_arccos_ufunc_npm(self):
-        self.test_arccos_ufunc(flags=no_pyobj_flags)
-
-    def test_arctan_ufunc(self, flags=enable_pyobj_flags):
+    def test_arctan_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.arctan, flags=flags, kinds='cf')
 
-    def test_arctan_ufunc_npm(self):
-        self.test_arctan_ufunc(flags=no_pyobj_flags)
-
-    def test_arctan2_ufunc(self, flags=enable_pyobj_flags):
+    def test_arctan2_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.arctan2, flags=flags, kinds='cf')
 
-    def test_arctan2_ufunc_npm(self):
-        self.test_arctan2_ufunc(flags=no_pyobj_flags)
-
-    def test_hypot_ufunc(self, flags=enable_pyobj_flags):
+    def test_hypot_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.hypot, kinds='f')
 
-    def test_hypot_ufunc_npm(self):
-        self.test_hypot_ufunc(flags=no_pyobj_flags)
-
-    def test_sinh_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_sinh_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.sinh, flags=flags, kinds='cf')
 
-    def test_sinh_ufunc_npm(self):
-        self.test_sinh_ufunc(flags=no_pyobj_flags)
-
-    def test_cosh_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_cosh_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.cosh, flags=flags, kinds='cf')
 
-    def test_cosh_ufunc_npm(self):
-        self.test_cosh_ufunc(flags=no_pyobj_flags)
-
-    def test_tanh_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_tanh_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.tanh, flags=flags, kinds='cf')
 
-    def test_tanh_ufunc_npm(self):
-        self.test_tanh_ufunc(flags=no_pyobj_flags)
-
-    def test_arcsinh_ufunc(self, flags=enable_pyobj_flags):
+    def test_arcsinh_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.arcsinh, flags=flags, kinds='cf')
 
-    def test_arcsinh_ufunc_npm(self):
-        self.test_arcsinh_ufunc(flags=no_pyobj_flags)
-
-    def test_arccosh_ufunc(self, flags=enable_pyobj_flags):
+    def test_arccosh_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.arccosh, flags=flags, kinds='cf')
 
-    def test_arccosh_ufunc_npm(self):
-        self.test_arccosh_ufunc(flags=no_pyobj_flags)
-
-    def test_arctanh_ufunc(self, flags=enable_pyobj_flags):
+    def test_arctanh_ufunc(self, flags=no_pyobj_flags):
         # arctanh is only valid is only finite in the range ]-1, 1[
         # This means that for any of the integer types it will produce
         # conversion from infinity/-infinity to integer. That's undefined
@@ -602,67 +508,36 @@ class TestUFuncs(BaseUFuncTest, TestCase):
         self.unary_ufunc_test(np.arctanh, skip_inputs=to_skip, flags=flags,
                               kinds='cf')
 
-    def test_arctanh_ufunc_npm(self):
-        self.test_arctanh_ufunc(flags=no_pyobj_flags)
-
-    def test_deg2rad_ufunc(self, flags=enable_pyobj_flags):
+    def test_deg2rad_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.deg2rad, flags=flags, kinds='f')
 
-    def test_deg2rad_ufunc_npm(self):
-        self.test_deg2rad_ufunc(flags=no_pyobj_flags)
-
-    def test_rad2deg_ufunc(self, flags=enable_pyobj_flags):
+    def test_rad2deg_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.rad2deg, flags=flags, kinds='f')
 
-    def test_rad2deg_ufunc_npm(self):
-        self.test_rad2deg_ufunc(flags=no_pyobj_flags)
-
-    def test_degrees_ufunc(self, flags=enable_pyobj_flags):
+    def test_degrees_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.degrees, flags=flags, kinds='f')
 
-    def test_degrees_ufunc_npm(self):
-        self.test_degrees_ufunc(flags=no_pyobj_flags)
-
-    def test_radians_ufunc(self, flags=enable_pyobj_flags):
+    def test_radians_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.radians, flags=flags, kinds='f')
 
-    def test_radians_ufunc_npm(self):
-        self.test_radians_ufunc(flags=no_pyobj_flags)
-
     ############################################################################
     # Bit-twiddling Functions
 
-    def test_bitwise_and_ufunc(self, flags=enable_pyobj_flags):
+    def test_bitwise_and_ufunc(self, flags=no_pyobj_flags):
         self.binary_int_ufunc_test(np.bitwise_and, flags=flags)
 
-    def test_bitwise_and_ufunc_npm(self):
-        self.test_bitwise_and_ufunc(flags=no_pyobj_flags)
-
-    def test_bitwise_or_ufunc(self, flags=enable_pyobj_flags):
+    def test_bitwise_or_ufunc(self, flags=no_pyobj_flags):
         self.binary_int_ufunc_test(np.bitwise_or, flags=flags)
 
-    def test_bitwise_or_ufunc_npm(self):
-        self.test_bitwise_or_ufunc(flags=no_pyobj_flags)
-
-    def test_bitwise_xor_ufunc(self, flags=enable_pyobj_flags):
+    def test_bitwise_xor_ufunc(self, flags=no_pyobj_flags):
         self.binary_int_ufunc_test(np.bitwise_xor, flags=flags)
 
-    def test_bitwise_xor_ufunc_npm(self):
-        self.test_bitwise_xor_ufunc(flags=no_pyobj_flags)
-
-    def test_invert_ufunc(self, flags=enable_pyobj_flags):
+    def test_invert_ufunc(self, flags=no_pyobj_flags):
         self.unary_int_ufunc_test(np.invert, flags=flags)
 
-    def test_invert_ufunc_npm(self):
-        self.test_invert_ufunc(flags=no_pyobj_flags)
-
-    def test_bitwise_not_ufunc(self, flags=enable_pyobj_flags):
+    def test_bitwise_not_ufunc(self, flags=no_pyobj_flags):
         self.unary_int_ufunc_test(np.bitwise_not, flags=flags)
 
-    def test_bitwise_not_ufunc_npm(self):
-        self.test_bitwise_not_ufunc(flags=no_pyobj_flags)
-
-
     # Note: there is no entry for left_shift and right_shift as this harness
     #       is not valid for them. This is so because left_shift and right
     #       shift implementation in NumPy has undefined behavior (in C-parlance)
@@ -675,171 +550,109 @@ class TestUFuncs(BaseUFuncTest, TestCase):
 
     ############################################################################
     # Comparison functions
-    def test_greater_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_greater_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.greater, flags=flags)
 
-    def test_greater_ufunc_npm(self):
-        self.test_greater_ufunc(flags=no_pyobj_flags)
-
-    def test_greater_equal_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_greater_equal_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.greater_equal, flags=flags)
 
-    def test_greater_equal_ufunc_npm(self):
-        self.test_greater_equal_ufunc(flags=no_pyobj_flags)
-
-    def test_less_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_less_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.less, flags=flags)
 
-    def test_less_ufunc_npm(self):
-        self.test_less_ufunc(flags=no_pyobj_flags)
-
-    def test_less_equal_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_less_equal_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.less_equal, flags=flags)
 
-    def test_less_equal_ufunc_npm(self):
-        self.test_less_equal_ufunc(flags=no_pyobj_flags)
-
-    def test_not_equal_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_not_equal_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.not_equal, flags=flags)
 
-    def test_not_equal_ufunc_npm(self):
-        self.test_not_equal_ufunc(flags=no_pyobj_flags)
-
-    def test_equal_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_equal_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.equal, flags=flags)
 
-    def test_equal_ufunc_npm(self):
-        self.test_equal_ufunc(flags=no_pyobj_flags)
-
-    def test_logical_and_ufunc(self, flags=enable_pyobj_flags):
+    def test_logical_and_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.logical_and, flags=flags)
 
-    def test_logical_and_ufunc_npm(self):
-        self.test_logical_and_ufunc(flags=no_pyobj_flags)
-
-    def test_logical_or_ufunc(self, flags=enable_pyobj_flags):
+    def test_logical_or_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.logical_or, flags=flags)
 
-    def test_logical_or_ufunc_npm(self):
-        self.test_logical_or_ufunc(flags=no_pyobj_flags)
-
-    def test_logical_xor_ufunc(self, flags=enable_pyobj_flags):
+    def test_logical_xor_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.logical_xor, flags=flags)
 
-    def test_logical_xor_ufunc_npm(self):
-        self.test_logical_xor_ufunc(flags=no_pyobj_flags)
-
-    def test_logical_not_ufunc(self, flags=enable_pyobj_flags):
+    def test_logical_not_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.logical_not, flags=flags)
 
-    def test_logical_not_ufunc_npm(self):
-        self.test_logical_not_ufunc(flags=no_pyobj_flags)
-
-    def test_maximum_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_maximum_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.maximum, flags=flags)
 
-    def test_maximum_ufunc_npm(self):
-        self.test_maximum_ufunc(flags=no_pyobj_flags)
-
-    def test_minimum_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_minimum_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.minimum, flags=flags)
 
-    def test_minimum_ufunc_npm(self):
-        self.test_minimum_ufunc(flags=no_pyobj_flags)
-
-    def test_fmax_ufunc(self, flags=enable_pyobj_flags):
+    def test_fmax_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.fmax, flags=flags)
 
-    def test_fmax_ufunc_npm(self):
-        self.test_fmax_ufunc(flags=no_pyobj_flags)
-
-    def test_fmin_ufunc(self, flags=enable_pyobj_flags):
+    def test_fmin_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.fmin, flags=flags)
 
-    def test_fmin_ufunc_npm(self):
-        self.test_fmin_ufunc(flags=no_pyobj_flags)
-
 
     ############################################################################
     # Floating functions
-    def test_isfinite_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_isfinite_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.isfinite, flags=flags)
 
-    def test_isfinite_ufunc_npm(self):
-        self.test_isfinite_ufunc(flags=no_pyobj_flags)
-
-    def test_isinf_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_isinf_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.isinf, flags=flags)
 
-    def test_isinf_ufunc_npm(self):
-        self.test_isinf_ufunc(flags=no_pyobj_flags)
-
-    def test_isnan_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_isnan_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.isnan, flags=flags)
 
-    def test_isnan_ufunc_npm(self):
-        self.test_isnan_ufunc(flags=no_pyobj_flags)
-
-    def test_signbit_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_signbit_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.signbit, flags=flags)
 
-    def test_signbit_ufunc_npm(self):
-        self.test_signbit_ufunc(flags=no_pyobj_flags)
-
-    def test_copysign_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_copysign_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.copysign, flags=flags, kinds='f')
 
-    def test_copysign_ufunc_npm(self):
-        self.test_copysign_ufunc(flags=no_pyobj_flags)
-
-    def test_nextafter_ufunc(self, flags=enable_pyobj_flags):
+    def test_nextafter_ufunc(self, flags=no_pyobj_flags):
         self.binary_ufunc_test(np.nextafter, flags=flags, kinds='f')
 
-    def test_nextafter_ufunc_npm(self):
-        self.test_nextafter_ufunc(flags=no_pyobj_flags)
-
-    def test_modf_ufunc(self, flags=enable_pyobj_flags):
-        self.unary_ufunc_test(np.modf, flags=flags, kinds='f')
-
     @_unimplemented
-    def test_modf_ufunc_npm(self):
-        self.test_modf_ufunc(flags=no_pyobj_flags)
+    def test_modf_ufunc(self, flags=no_pyobj_flags):
+        self.unary_ufunc_test(np.modf, flags=flags, kinds='f')
 
     # Note: there is no entry for ldexp as this harness isn't valid for this
     #       ufunc. this is so because ldexp requires heterogeneous inputs.
     #       However, this ufunc is tested by the TestLoopTypes test classes.
 
-    def test_frexp_ufunc(self, flags=enable_pyobj_flags):
+    @_unimplemented
+    def test_frexp_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.frexp, flags=flags, kinds='f')
 
-    @_unimplemented
-    def test_frexp_ufunc_npm(self):
-        self.test_frexp_ufunc(flags=no_pyobj_flags)
-
-    def test_floor_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_floor_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.floor, flags=flags, kinds='f')
 
-    def test_floor_ufunc_npm(self):
-        self.test_floor_ufunc(flags=no_pyobj_flags)
-
-    def test_ceil_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_ceil_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.ceil, flags=flags, kinds='f')
 
-    def test_ceil_ufunc_npm(self):
-        self.test_ceil_ufunc(flags=no_pyobj_flags)
-
-    def test_trunc_ufunc(self, flags=enable_pyobj_flags):
+    @tag('important')
+    def test_trunc_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.trunc, flags=flags, kinds='f')
 
-    def test_trunc_ufunc_npm(self):
-        self.test_trunc_ufunc(flags=no_pyobj_flags)
-
-    def test_spacing_ufunc(self, flags=enable_pyobj_flags):
+    def test_spacing_ufunc(self, flags=no_pyobj_flags):
         self.unary_ufunc_test(np.spacing, flags=flags, kinds='f')
 
-    def test_spacing_ufunc_npm(self):
-        self.test_spacing_ufunc(flags=no_pyobj_flags)
-
     ############################################################################
     # Other tests
     def test_binary_ufunc_performance(self):
@@ -865,7 +678,7 @@ class TestUFuncs(BaseUFuncTest, TestCase):
         print(utils.benchmark(bm_numba, maxsec=.1))
         assert np.allclose(control, result)
 
-    def binary_ufunc_mixed_types_test(self, ufunc, flags=enable_pyobj_flags):
+    def binary_ufunc_mixed_types_test(self, ufunc, flags=no_pyobj_flags):
         ufunc_name = ufunc.__name__
         ufunc = _make_binary_ufunc_usecase(ufunc)
         inputs1 = [
@@ -946,6 +759,7 @@ class TestUFuncs(BaseUFuncTest, TestCase):
         if not numpy_support.strict_ufunc_typing:
             self.binary_ufunc_mixed_types_test(np.divide, flags=no_pyobj_flags)
 
+    @tag('important')
     def test_broadcasting(self):
 
         # Test unary ufunc
@@ -1245,6 +1059,7 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
     def test_unary_positive_array_op(self):
         self.unary_op_test('+')
 
+    @tag('important')
     def test_unary_negative_array_op(self):
         self.unary_op_test('-')
 
@@ -1257,12 +1072,15 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
     # ____________________________________________________________
     # Inplace operators
 
+    @tag('important')
     def test_inplace_add(self):
         self.inplace_float_op_test('+=', [-1, 1.5, 3], [-5, 0, 2.5])
 
+    @tag('important')
     def test_inplace_sub(self):
         self.inplace_float_op_test('-=', [-1, 1.5, 3], [-5, 0, 2.5])
 
+    @tag('important')
     def test_inplace_mul(self):
         self.inplace_float_op_test('*=', [-1, 1.5, 3], [-5, 0, 2.5])
 
@@ -1275,6 +1093,7 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
     def test_inplace_remainder(self):
         self.inplace_float_op_test('%=', [-1, 1.5, 3], [-5, 2, 2.5])
 
+    @tag('important')
     def test_inplace_pow(self):
         self.inplace_float_op_test('**=', [-1, 1.5, 3], [-5, 2, 2.5])
 
@@ -1287,6 +1106,7 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
     def test_inplace_xor(self):
         self.inplace_int_op_test('^=', [0, 1, 2, 3, 51], [0, 13, 16, 42, 255])
 
+    @tag('important')
     def test_inplace_lshift(self):
         self.inplace_int_op_test('<<=', [0, 5, -10, -51], [0, 1, 4, 14])
 
@@ -1318,60 +1138,78 @@ class TestArrayOperators(BaseUFuncTest, TestCase):
     # ____________________________________________________________
     # Binary operators
 
+    @tag('important')
     def test_add_array_op(self):
         self.binary_op_test('+')
 
+    @tag('important')
     def test_subtract_array_op(self):
         self.binary_op_test('-')
 
+    @tag('important')
     def test_multiply_array_op(self):
         self.binary_op_test('*')
 
+    @tag('important')
     def test_divide_array_op(self):
         int_out_type = None
         if PYVERSION >= (3, 0):
             int_out_type = types.float64
         self.binary_op_test('/', int_output_type=int_out_type)
 
+    @tag('important')
     def test_floor_divide_array_op(self):
         self.binary_op_test('//')
 
+    @tag('important')
     def test_remainder_array_op(self):
         self.binary_op_test('%')
 
+    @tag('important')
     def test_power_array_op(self):
         self.binary_op_test('**')
 
+    @tag('important')
     def test_left_shift_array_op(self):
         self.binary_int_op_test('<<', positive_rhs=True)
 
+    @tag('important')
     def test_right_shift_array_op(self):
         self.binary_int_op_test('>>', positive_rhs=True)
 
+    @tag('important')
     def test_bitwise_and_array_op(self):
         self.binary_int_op_test('&')
 
+    @tag('important')
     def test_bitwise_or_array_op(self):
         self.binary_int_op_test('|')
 
+    @tag('important')
     def test_bitwise_xor_array_op(self):
         self.binary_int_op_test('^')
 
+    @tag('important')
     def test_equal_array_op(self):
         self.binary_op_test('==')
 
+    @tag('important')
     def test_greater_array_op(self):
         self.binary_op_test('>')
 
+    @tag('important')
     def test_greater_equal_array_op(self):
         self.binary_op_test('>=')
 
+    @tag('important')
     def test_less_array_op(self):
         self.binary_op_test('<')
 
+    @tag('important')
     def test_less_equal_array_op(self):
         self.binary_op_test('<=')
 
+    @tag('important')
     def test_not_equal_array_op(self):
         self.binary_op_test('!=')
 
@@ -1493,13 +1331,9 @@ class TestUfuncIssues(TestCase):
         self.assertEqual(foo(1j, 1j), cr.entry_point(1j, 1j))
 
 
-class _TestLoopTypes(TestCase):
+class _LoopTypesTester(TestCase):
     """Test code generation for the different loop types defined by ufunc.
 
-    This class tests the ufuncs without forcing no-python mode. Subclasses
-    of this class tweak it so they tests no-python mode support for the
-    different ufuncs.
-
     This test relies on class variables to configure the test. Subclasses
     of this class can just override some of these variables to check other
     ufuncs in a different compilation context. The variables supported are:
@@ -1519,20 +1353,8 @@ class _TestLoopTypes(TestCase):
     you can get a list of supported signatures by accessing the attribute
     'types'.
     """
-
-    _ufuncs = all_ufuncs[:]
-    # Have their own test classes
-    _ufuncs.remove(np.left_shift)
-    _ufuncs.remove(np.right_shift)
-    _ufuncs.remove(np.reciprocal)
-    _ufuncs.remove(np.power)
-    _compile_flags = enable_pyobj_flags
     _skip_types = 'OegG'
 
-    # Skip datetime64 'M' and timedelta64 'm' on numpy 1.6
-    if is_on_numpy_16:
-        _skip_types += 'Mm'
-
     _ulps = {('arccos', 'F'): 2,
              ('tanh', 'F'): 2,
              }
@@ -1648,13 +1470,7 @@ class _TestLoopTypes(TestCase):
             cls._check_ufunc_loops(ufunc)
 
 
-class TestLoopTypes(_TestLoopTypes):
-    pass
-
-TestLoopTypes.autogenerate()
-
-
-class TestLoopTypesIntNoPython(_TestLoopTypes):
+class TestLoopTypesIntNoPython(_LoopTypesTester):
     _compile_flags = no_pyobj_flags
     _ufuncs = supported_ufuncs[:]
     # reciprocal and power need a special test due to issue #757
@@ -1663,15 +1479,16 @@ class TestLoopTypesIntNoPython(_TestLoopTypes):
     _ufuncs.remove(np.left_shift) # has its own test class
     _ufuncs.remove(np.right_shift) # has its own test class
     _required_types = '?bBhHiIlLqQ'
-    _skip_types = 'fdFDmMO' + _TestLoopTypes._skip_types
+    _skip_types = 'fdFDmMO' + _LoopTypesTester._skip_types
 
 TestLoopTypesIntNoPython.autogenerate()
 
-class TestLoopTypesReciprocalNoPython(_TestLoopTypes):
+
+class TestLoopTypesReciprocalNoPython(_LoopTypesTester):
     _compile_flags = no_pyobj_flags
     _ufuncs = [np.reciprocal] # issue #757
     _required_types = 'bBhHiIlLqQfdFD'
-    _skip_types = 'mMO' + _TestLoopTypes._skip_types
+    _skip_types = 'mMO' + _LoopTypesTester._skip_types
 
     def _arg_for_type(self, a_letter_type, index=0):
         res = super(self.__class__, self)._arg_for_type(a_letter_type,
@@ -1685,11 +1502,12 @@ class TestLoopTypesReciprocalNoPython(_TestLoopTypes):
 
 TestLoopTypesReciprocalNoPython.autogenerate()
 
-class TestLoopTypesPowerNoPython(_TestLoopTypes):
+
+class TestLoopTypesPowerNoPython(_LoopTypesTester):
     _compile_flags = no_pyobj_flags
     _ufuncs = [np.power] # issue #757
     _required_types = 'bBhHiIlLqQfdFD'
-    _skip_types = 'mMO' + _TestLoopTypes._skip_types
+    _skip_types = 'mMO' + _LoopTypesTester._skip_types
 
     def _arg_for_type(self, a_letter_type, index=0):
         res = super(self.__class__, self)._arg_for_type(a_letter_type,
@@ -1703,11 +1521,12 @@ class TestLoopTypesPowerNoPython(_TestLoopTypes):
 
 TestLoopTypesPowerNoPython.autogenerate()
 
-class TestLoopTypesIntLeftShiftNoPython(_TestLoopTypes):
+
+class TestLoopTypesIntLeftShiftNoPython(_LoopTypesTester):
     _compile_flags = no_pyobj_flags
     _ufuncs = [np.left_shift]
     _required_types = 'bBhHiIlLqQ'
-    _skip_types = 'fdFDmMO' + _TestLoopTypes._skip_types
+    _skip_types = 'fdFDmMO' + _LoopTypesTester._skip_types
 
     def _arg_for_type(self, a_letter_type, index=0):
         res = super(self.__class__, self)._arg_for_type(a_letter_type,
@@ -1725,11 +1544,12 @@ class TestLoopTypesIntLeftShiftNoPython(_TestLoopTypes):
 
 TestLoopTypesIntLeftShiftNoPython.autogenerate()
 
-class TestLoopTypesIntRightShiftNoPython(_TestLoopTypes):
+
+class TestLoopTypesIntRightShiftNoPython(_LoopTypesTester):
     _compile_flags = no_pyobj_flags
     _ufuncs = [np.right_shift]
     _required_types = 'bBhHiIlLqQ'
-    _skip_types = 'fdFDmMO' + _TestLoopTypes._skip_types
+    _skip_types = 'fdFDmMO' + _LoopTypesTester._skip_types
 
     def _arg_for_type(self, a_letter_type, index=0):
         res = super(self.__class__, self)._arg_for_type(a_letter_type,
@@ -1758,29 +1578,32 @@ class TestLoopTypesIntRightShiftNoPython(_TestLoopTypes):
 
 TestLoopTypesIntRightShiftNoPython.autogenerate()
 
-class TestLoopTypesFloatNoPython(_TestLoopTypes):
+
+class TestLoopTypesFloatNoPython(_LoopTypesTester):
     _compile_flags = no_pyobj_flags
     _ufuncs = supported_ufuncs[:]
     if iswindows:
         _ufuncs.remove(np.signbit) # TODO: fix issue #758
     _required_types = 'fd'
-    _skip_types = 'FDmMO' + _TestLoopTypes._skip_types
+    _skip_types = 'FDmMO' + _LoopTypesTester._skip_types
 
 TestLoopTypesFloatNoPython.autogenerate()
 
-class TestLoopTypesComplexNoPython(_TestLoopTypes):
+
+class TestLoopTypesComplexNoPython(_LoopTypesTester):
     _compile_flags = no_pyobj_flags
     _ufuncs = supported_ufuncs[:]
 
     # Test complex types
     # Every loop containing a complex argument must be tested
     _required_types = 'FD'
-    _skip_types = 'mMO' + _TestLoopTypes._skip_types
+    _skip_types = 'mMO' + _LoopTypesTester._skip_types
 
 TestLoopTypesComplexNoPython.autogenerate()
 
+
 @skip_on_numpy_16
-class TestLoopTypesDatetimeNoPython(_TestLoopTypes):
+class TestLoopTypesDatetimeNoPython(_LoopTypesTester):
     _compile_flags = no_pyobj_flags
     _ufuncs = supported_ufuncs[:]
 
@@ -1878,6 +1701,7 @@ class TestLoopTypesDatetimeNoPython(_TestLoopTypes):
 
 TestLoopTypesDatetimeNoPython.autogenerate()
 
+
 class TestUFuncBadArgsNoPython(TestCase):
     _compile_flags = no_pyobj_flags
 
