@@ -107,17 +107,20 @@ class ConstraintNetwork(object):
         """
         errors = []
         for constraint in self.constraints:
-            try:
-                constraint(typeinfer)
-            except TypingError as e:
-                errors.append(e)
-            except Exception:
-                msg = "Internal error at {con}:\n{sep}\n{err}{sep}\n"
-                e = TypingError(msg.format(con=constraint,
-                                           err=traceback.format_exc(),
-                                           sep='--%<' +'-' * 65),
-                                loc=constraint.loc)
-                errors.append(e)
+            loc = constraint.loc
+            with typeinfer.warnings.catch_warnings(filename=loc.filename,
+                                                   lineno=loc.line):
+                try:
+                    constraint(typeinfer)
+                except TypingError as e:
+                    errors.append(e)
+                except Exception:
+                    msg = "Internal error at {con}:\n{sep}\n{err}{sep}\n"
+                    e = TypingError(msg.format(con=constraint,
+                                               err=traceback.format_exc(),
+                                               sep='--%<' +'-' * 65),
+                                    loc=constraint.loc)
+                    errors.append(e)
         return errors
 
 
@@ -142,6 +145,24 @@ class Propagate(object):
         typeinfer.add_type(self.src, target_type, unless_locked=True)
 
 
+class ArgConstraint(object):
+
+    def __init__(self, dst, src, loc):
+        self.dst = dst
+        self.src = src
+        self.loc = loc
+
+    def __call__(self, typeinfer):
+        typevars = typeinfer.typevars
+        src = typevars[self.src]
+        if not src.defined:
+            return
+        ty = src.getone()
+        if isinstance(ty, types.Omitted):
+            ty = typeinfer.context.resolve_value_type(ty.value)
+        typeinfer.add_type(self.dst, ty)
+
+
 class BuildTupleConstraint(object):
     def __init__(self, target, items, loc):
         self.target = target
@@ -495,7 +516,7 @@ class TypeInferer(object):
     Operates on block that shares the same ir.Scope.
     """
 
-    def __init__(self, context, interp):
+    def __init__(self, context, interp, warnings):
         self.context = context
         self.blocks = interp.blocks
         self.generator_info = interp.generator_info
@@ -503,6 +524,7 @@ class TypeInferer(object):
         self.typevars = TypeVarMap()
         self.typevars.set_context(context)
         self.constraints = ConstraintNetwork()
+        self.warnings = warnings
 
         # { index: mangled name }
         self.arg_names = {}
@@ -569,8 +591,10 @@ class TypeInferer(object):
         tv = self.typevars[var]
         if unless_locked and tv.locked:
             return
+        oldty = tv.type
         unified = tv.add_type(tp)
-        self.propagate_refined_type(var, unified)
+        if unified != oldty:
+            self.propagate_refined_type(var, unified)
 
     def add_calltype(self, inst, signature):
         self.calltypes[inst] = signature
@@ -743,9 +767,9 @@ class TypeInferer(object):
 
     def typeof_arg(self, inst, target, arg):
         src_name = self._mangle_arg_name(arg.name)
-        self.constraints.append(Propagate(dst=target.name,
-                                          src=src_name,
-                                          loc=inst.loc))
+        self.constraints.append(ArgConstraint(dst=target.name,
+                                              src=src_name,
+                                              loc=inst.loc))
 
     def typeof_const(self, inst, target, const):
         self.lock_type(target.name, self.resolve_value_type(inst, const))
