@@ -167,11 +167,33 @@ class Const(Dummy):
 
     def __init__(self, value):
         self.value = value
+        # We want to support constants of non-hashable values, therefore
+        # fall back on the value's id() if necessary.
+        try:
+            hash(value)
+        except TypeError:
+            self._key = id(value)
+        else:
+            self._key = value
         super(Const, self).__init__("const(%r)" % (value,))
 
     @property
     def key(self):
-        return type(self.value), self.value
+        return type(self.value), self._key
+
+
+class Omitted(Opaque):
+    """
+    An omitted function argument with a default value.
+    """
+
+    def __init__(self, value):
+        self.value = value
+        super(Omitted, self).__init__("omitted(default=%r)" % (value,))
+
+    @property
+    def key(self):
+        return type(self.value), id(self.value)
 
 
 class VarArg(Type):
@@ -420,9 +442,14 @@ class Dispatcher(WeakType, Callable, Dummy):
         super(Dispatcher, self).__init__("Dispatcher(%s)" % dispatcher)
 
     def get_call_type(self, context, args, kws):
-        template, args, kws = self.dispatcher.get_call_template(args, kws)
+        """
+        Resolve a call to this dispatcher using the given argument types.
+        A signature returned and it is ensured that a compiled specialization
+        is available for it.
+        """
+        template, pysig, args, kws = self.dispatcher.get_call_template(args, kws)
         sig = template(context).apply(args, kws)
-        sig.pysig = self.pysig
+        sig.pysig = pysig
         return sig
 
     def get_call_signatures(self):
@@ -436,13 +463,6 @@ class Dispatcher(WeakType, Callable, Dummy):
         """
         return self._get_object()
 
-    @property
-    def pysig(self):
-        """
-        A inspect.Signature object corresponding to this type.
-        """
-        return self.dispatcher._pysig
-
     def get_overload(self, sig):
         """
         Get the compiled overload for the given signature.
@@ -950,6 +970,24 @@ class Array(Buffer):
                 and (self.aligned or not other.aligned)):
                 return Conversion.safe
 
+class SmartArrayType(Array):
+
+    def __init__(self, dtype, ndim, layout, pyclass):
+        self.pyclass = pyclass
+        super(SmartArrayType, self).__init__(dtype, ndim, layout, name='numba_array')
+
+    @property
+    def as_array(self):
+        return Array(self.dtype, self.ndim, self.layout)
+
+    def copy(self, dtype=None, ndim=None, layout=None):
+        if dtype is None:
+            dtype = self.dtype
+        if ndim is None:
+            ndim = self.ndim
+        if layout is None:
+            layout = self.layout
+        return type(self)(dtype, ndim, layout, self.pyclass)
 
 class ArrayCTypes(Type):
     """
