@@ -6,6 +6,9 @@ from numba import types
 from numba.typing.templates import (AttributeTemplate, AbstractTemplate,
                                     infer, infer_getattr, signature,
                                     bound_function)
+# import time side effect: array operations requires typing support of sequence
+# defined in collections: e.g. array.shape[i]
+from numba.typing import collections
 
 Indexing = namedtuple("Indexing", ("index", "result", "advanced"))
 
@@ -176,17 +179,29 @@ class SetItemBuffer(AbstractTemplate):
         res = out.result
         if isinstance(res, types.Array):
             # Indexing produces an array
-            if not isinstance(val, types.Array):
-                # Allow scalar broadcasting
-                res = res.dtype
-            elif (val.ndim == res.ndim and
-                  self.context.can_convert(val.dtype, res.dtype)):
-                # Allow assignement of same-dimensionality compatible-dtype array
-                res = val
+            if isinstance(val, types.Array):
+                if (val.ndim == res.ndim and
+                    self.context.can_convert(val.dtype, res.dtype)):
+                    # Allow assignement of same-dimensionality compatible-dtype array
+                    res = val
+                else:
+                    # NOTE: array broadcasting is unsupported
+                    return
+            elif isinstance(val, types.Sequence):
+                if (res.ndim == 1 and
+                    self.context.can_convert(val.dtype, res.dtype)):
+                    # Allow assignement of sequence to 1d array
+                    res = val
+                else:
+                    # NOTE: sequence-to-array broadcasting is unsupported
+                    return
             else:
-                # Unexpected dimensionality of assignment source
-                # (array broadcasting is unsupported)
-                return
+                # Allow scalar broadcasting
+                if self.context.can_convert(val, res.dtype):
+                    res = res.dtype
+                else:
+                    # Incompatible scalar type
+                    return
         elif not isinstance(val, types.Array):
             # Single item assignment
             res = val
@@ -325,6 +340,20 @@ class ArrayAttribute(AttributeTemplate):
         retty = ary.copy(dtype=dtype)
         return signature(retty, *args)
 
+    @bound_function("array.ravel")
+    def resolve_ravel(self, ary, args, kws):
+        # Only support no argument version (default order='C')
+        assert not kws
+        assert not args
+        return signature(ary.copy(ndim=1, layout='C'))
+
+    @bound_function("array.flatten")
+    def resolve_flatten(self, ary, args, kws):
+        # Only support no argument version (default order='C')
+        assert not kws
+        assert not args
+        return signature(ary.copy(ndim=1, layout='C'))
+
     def generic_resolve(self, ary, attr):
         # Resolution of other attributes, for record arrays
         if isinstance(ary.dtype, types.Record):
@@ -415,6 +444,8 @@ def _expand_integer(ty):
             return max(types.intp, ty)
         else:
             return max(types.uintp, ty)
+    elif isinstance(ty, types.Boolean):
+        return types.intp
     else:
         return ty
 
@@ -435,7 +466,7 @@ def generic_expand_cumulative(self, args, kws):
 def generic_hetero_real(self, args, kws):
     assert not args
     assert not kws
-    if self.this.dtype in types.integer_domain:
+    if isinstance(self.this.dtype, (types.Integer, types.Boolean)):
         return signature(types.float64, recvr=self.this)
     return signature(self.this.dtype, recvr=self.this)
 
