@@ -2,7 +2,10 @@ from __future__ import print_function, division, absolute_import
 
 import itertools
 
-from numba import types, intrinsics
+import numpy
+
+from numba import types
+
 from numba.utils import PYVERSION, RANGE_ITER_OBJECTS, operator_map
 from numba.typing.templates import (AttributeTemplate, ConcreteTemplate,
                                     AbstractTemplate, infer_global, infer,
@@ -484,8 +487,12 @@ class StaticGetItemTuple(AbstractTemplate):
 
     def generic(self, args, kws):
         tup, idx = args
-        if isinstance(tup, types.BaseTuple) and isinstance(idx, int):
+        if not isinstance(tup, types.BaseTuple):
+            return
+        if isinstance(idx, int):
             return tup.types[idx]
+        elif isinstance(idx, slice):
+            return types.BaseTuple.from_types(tup.types[idx])
 
 
 #-------------------------------------------------------------------------------
@@ -582,7 +589,14 @@ class NumberClassAttribute(AttributeTemplate):
         ty = classty.instance_type
 
         def typer(val):
-            return ty
+            if isinstance(val, (types.BaseTuple, types.Sequence)):
+                # Array constructor, e.g. np.int32([1, 2])
+                sig = self.context.resolve_function_type(
+                    numpy.array, (val,), {'dtype': types.DType(ty)})
+                return sig.return_type
+            else:
+                # Scalar constructor, e.g. np.int32(42)
+                return ty
 
         return types.Function(make_callable_template(key=ty, typer=typer))
 
@@ -762,18 +776,6 @@ class Zip(AbstractTemplate):
             zip_type = types.ZipType(args)
             return signature(zip_type, *args)
 
-
-@infer_global(intrinsics.array_ravel)
-class Intrinsic_array_ravel(AbstractTemplate):
-    key = intrinsics.array_ravel
-
-    def generic(self, args, kws):
-        assert not kws
-        [arr] = args
-        if arr.layout in 'CF' and arr.ndim >= 1:
-            return signature(arr.copy(ndim=1), arr)
-
-
 #------------------------------------------------------------------------------
 
 @infer_global(type)
