@@ -9,12 +9,7 @@ from numba.typeconv import Conversion, rules
 from . import templates
 from .typeof import typeof, Purpose
 
-# Initialize declarations
-from . import (
-    builtins, arraydecl, cmathdecl, listdecl, mathdecl, npdatetime, npydecl,
-    operatordecl, randomdecl)
 from numba import utils
-from . import ctypes_utils, cffi_utils, bufproto
 
 
 class Rating(object):
@@ -66,8 +61,9 @@ class BaseContext(object):
         Refresh context with new declarations from known registries.
         Useful for third-party extensions.
         """
-        self._load_builtins()
         self.load_additional_registries()
+        # Some extensions may have augmented the builtin registry
+        self._load_builtins()
 
     def explain_function_type(self, func):
         """
@@ -105,11 +101,18 @@ class BaseContext(object):
         Resolve function type *func* for argument types *args* and *kws*.
         A signature is returned.
         """
-        defns = self._functions[func]
-        for defn in defns:
-            res = defn.apply(args, kws)
-            if res is not None:
-                return res
+        if func not in self._functions:
+            # It's not a known function type, perhaps it's a global?
+            try:
+                func = self._lookup_global(func)
+            except KeyError:
+                pass
+        if func in self._functions:
+            defns = self._functions[func]
+            for defn in defns:
+                res = defn.apply(args, kws)
+                if res is not None:
+                    return res
 
         if isinstance(func, types.Type):
             # If it's a type, it may support a __call__ method
@@ -242,6 +245,9 @@ class BaseContext(object):
                 raise
 
     def _load_builtins(self):
+        # Initialize declarations
+        from . import builtins, arraydecl, npdatetime
+        from . import ctypes_utils, bufproto
         self.install_registry(templates.builtin_registry)
 
     def load_additional_registries(self):
@@ -459,7 +465,7 @@ class BaseContext(object):
         unified = typelist[0]
         for tp in typelist[1:]:
             unified = self.unify_pairs(unified, tp)
-            if unified is None:
+            if unified is types.pyobject:
                 break
         return unified
 
@@ -503,10 +509,12 @@ class BaseContext(object):
 class Context(BaseContext):
 
     def load_additional_registries(self):
+        from . import (cffi_utils, cmathdecl, listdecl, mathdecl,
+                       npydecl, operatordecl, randomdecl)
+        self.install_registry(cffi_utils.registry)
         self.install_registry(cmathdecl.registry)
         self.install_registry(listdecl.registry)
         self.install_registry(mathdecl.registry)
         self.install_registry(npydecl.registry)
         self.install_registry(operatordecl.registry)
         self.install_registry(randomdecl.registry)
-        self.install_registry(cffi_utils.registry)
