@@ -1,16 +1,19 @@
 from __future__ import absolute_import, print_function
 
+import warnings
+
 import numpy
+
 from .. import types, utils
 from .templates import (AttributeTemplate, AbstractTemplate, CallableTemplate,
                         Registry, signature)
 
 from ..numpy_support import (ufunc_find_matching_loop,
                              supported_ufunc_loop, as_dtype,
-                             from_dtype, resolve_output_type)
+                             from_dtype, as_dtype, resolve_output_type)
 from ..numpy_support import version as numpy_version
-
 from ..errors import TypingError
+from ..config import PerformanceWarning
 
 registry = Registry()
 infer = registry.register
@@ -197,17 +200,16 @@ class NumpyRulesArrayOperator(Numpy_rules_ufunc):
         '''
         try:
             sig = super(NumpyRulesArrayOperator, self).generic(args, kws)
-            # Stay out of the timedelta64 range and domain; already
-            # handled elsewhere.
-            if sig is not None:
-                timedelta_test = (
-                    isinstance(sig.return_type, types.NPTimedelta) or
-                    (all(isinstance(argty, types.NPTimedelta)
-                         for argty in sig.args)))
-                if timedelta_test:
-                    sig = None
         except TypingError:
-            sig = None
+            return None
+        if sig is None:
+            return None
+        args = sig.args
+        # Only accept at least one array argument, otherwise the operator
+        # doesn't involve Numpy's ufunc machinery.
+        if not any(isinstance(arg, types.ArrayCompatible)
+                   for arg in args):
+            return None
         return sig
 
 
@@ -364,7 +366,7 @@ def _numpy_redirect(fname):
     infer_global(numpy_function, types.Function(cls))
 
 for func in ['min', 'max', 'sum', 'prod', 'mean', 'median', 'var', 'std',
-             'cumsum', 'cumprod', 'argmin', 'argmax', 'nonzero']:
+             'cumsum', 'cumprod', 'argmin', 'argmax', 'nonzero', 'ravel']:
     _numpy_redirect(func)
 
 
@@ -407,6 +409,59 @@ def _parse_dtype(dtype):
     if isinstance(dtype, types.DTypeSpec):
         return dtype.dtype
 
+def _parse_nested_sequence(context, typ):
+    """
+    Parse a (possibly 0d) nested sequence type.
+    A (ndim, dtype) tuple is returned.  Note the sequence may still be
+    heterogenous, as long as it converts to the given dtype.
+    """
+    if isinstance(typ, (types.Buffer,)):
+        raise TypingError("%r not allowed in a homogenous sequence")
+    elif isinstance(typ, (types.Sequence,)):
+        n, dtype = _parse_nested_sequence(context, typ.dtype)
+        return n + 1, dtype
+    elif isinstance(typ, (types.BaseTuple,)):
+        if typ.count == 0:
+            # Mimick Numpy's behaviour
+            return 1, types.float64
+        n, dtype = _parse_nested_sequence(context, typ[0])
+        dtypes = [dtype]
+        for i in range(1, typ.count):
+            _n, dtype = _parse_nested_sequence(context, typ[i])
+            if _n != n:
+                raise TypingError("type %r does not have a regular shape"
+                                  % (typ,))
+            dtypes.append(dtype)
+        dtype = context.unify_types(*dtypes)
+        if dtype is types.pyobject:
+            raise TypingError("cannot convert %r to a homogenous type")
+        return n + 1, dtype
+    else:
+        # Scalar type => check it's valid as a Numpy array dtype
+        as_dtype(typ)
+        return 0, typ
+
+
+
+@infer_global(numpy.array)
+class NpArray(CallableTemplate):
+    """
+    Typing template for np.array().
+    """
+
+    def generic(self):
+        def typer(object, dtype=None):
+            ndim, seq_dtype = _parse_nested_sequence(self.context, object)
+            if dtype is None:
+                dtype = seq_dtype
+            else:
+                dtype = _parse_dtype(dtype)
+                if dtype is None:
+                    return
+            return types.Array(dtype, ndim, 'C')
+
+        return typer
+
 
 @infer_global(numpy.empty)
 @infer_global(numpy.zeros)
@@ -657,10 +712,8 @@ class MatMulTyperMixin(object):
             all_args = (a, b)
 
         if not all(x.layout in 'CF' for x in (a, b)):
-            # Numpy seems to allow non-contiguous arguments, but we
-            # don't (we would need to copy before calling BLAS)
-            raise TypingError("%s only supported on "
-                              "contiguous arrays" % (self.func_name,))
+            warnings.warn("%s is faster on contiguous arrays, called on %s"
+                          % (self.func_name, (a, b)), PerformanceWarning)
         if not all(x.dtype == a.dtype for x in all_args):
             raise TypingError("%s arguments must all have "
                               "the same dtype" % (self.func_name,))
@@ -699,10 +752,8 @@ class VDot(CallableTemplate):
             if not all(x.ndim == 1 for x in (a, b)):
                 raise TypingError("np.vdot() only supported on 1-D arrays")
             if not all(x.layout in 'CF' for x in (a, b)):
-                # Numpy seems to allow non-contiguous arguments, but we
-                # don't (we would need to copy before calling BLAS)
-                raise TypingError("np.vdot() only supported on "
-                                  "contiguous arrays")
+                warnings.warn("np.vdot() is faster on contiguous arrays, called on %s"
+                              % ((a, b),), PerformanceWarning)
             if not all(x.dtype == a.dtype for x in (a, b)):
                 raise TypingError("np.vdot() arguments must all have "
                                   "the same dtype")
@@ -726,6 +777,23 @@ class MatMul(MatMulTyperMixin, AbstractTemplate):
             return signature(restype, *args)
 
 
+@infer_global(numpy.linalg.inv)
+class LinalgInv(CallableTemplate):
+
+    def generic(self):
+        def typer(a):
+            if not isinstance(a, types.Array):
+                return
+            if not a.ndim == 2:
+                raise TypingError("np.linalg.inv() only supported on 2-D arrays")
+            if not isinstance(a.dtype, (types.Float, types.Complex)):
+                raise TypingError("np.linalg.inv() only supported on "
+                                  "float and complex arrays")
+            return a.copy(layout='C')
+
+        return typer
+
+
 # -----------------------------------------------------------------------------
 # Miscellaneous functions
 
@@ -845,9 +913,37 @@ class Angle(CallableTemplate):
     Typing template for np.angle()
     """
     def generic(self):
-        def typer(ref, deg = False):
-            if isinstance(ref, types.Array):
-                return ref.copy(dtype=ref.underlying_float)
+        def typer(z, deg=False):
+            if isinstance(z, types.Array):
+                dtype = z.dtype
             else:
-                return types.float64
+                dtype = z
+            if isinstance(dtype, types.Complex):
+                ret_dtype = dtype.underlying_float
+            elif isinstance(dtype, types.Float):
+                ret_dtype = dtype
+            else:
+                return
+            if isinstance(z, types.Array):
+                return z.copy(dtype=ret_dtype)
+            else:
+                return ret_dtype
+        return typer
+
+
+@infer_global(numpy.diag)
+class DiagCtor(CallableTemplate):
+    """
+    Typing template for np.diag()
+    """
+    def generic(self):
+        def typer(ref, k=0):
+            if isinstance(ref, types.Array):
+                if ref.ndim == 1:
+                    rdim = 2
+                elif ref.ndim == 2:
+                    rdim = 1
+                else:
+                    return None
+                return types.Array(ndim=rdim, dtype=ref.dtype, layout='C')
         return typer
